# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Examples of Python implementations of the stock.proto Stock service."""

from grpc.framework.common import cardinality
from grpc.framework.face.testing import service
from grpc.framework.foundation import abandonment
from grpc.framework.foundation import stream
from grpc.framework.foundation import stream_util
from grpc._junkdrawer import stock_pb2

SYMBOL_FORMAT = 'test symbol:%03d'
STREAM_LENGTH = 400

# A test-appropriate security-pricing function. :-P
_price = lambda symbol_name: float(hash(symbol_name) % 4096)


def _get_last_trade_price(stock_request, stock_reply_callback, control, active):
  """A unary-request, unary-response test method."""
  control.control()
  if active():
    stock_reply_callback(
        stock_pb2.StockReply(
            symbol=stock_request.symbol, price=_price(stock_request.symbol)))
  else:
    raise abandonment.Abandoned()


def _get_last_trade_price_multiple(stock_reply_consumer, control, active):
  """A stream-request, stream-response test method."""
  def stock_reply_for_stock_request(stock_request):
    control.control()
    if active():
      return stock_pb2.StockReply(
          symbol=stock_request.symbol, price=_price(stock_request.symbol))
    else:
      raise abandonment.Abandoned()
  return stream_util.TransformingConsumer(
      stock_reply_for_stock_request, stock_reply_consumer)


def _watch_future_trades(stock_request, stock_reply_consumer, control, active):
  """A unary-request, stream-response test method."""
  base_price = _price(stock_request.symbol)
  for index in range(stock_request.num_trades_to_watch):
    control.control()
    if active():
      stock_reply_consumer.consume(
          stock_pb2.StockReply(
              symbol=stock_request.symbol, price=base_price + index))
    else:
      raise abandonment.Abandoned()
  stock_reply_consumer.terminate()


def _get_highest_trade_price(stock_reply_callback, control, active):
  """A stream-request, unary-response test method."""

  class StockRequestConsumer(stream.Consumer):
    """Keeps an ongoing record of the most valuable symbol yet consumed."""

    def __init__(self):
      self._symbol = None
      self._price = None

    def consume(self, stock_request):
      control.control()
      if active():
        if self._price is None:
          self._symbol = stock_request.symbol
          self._price = _price(stock_request.symbol)
        else:
          candidate_price = _price(stock_request.symbol)
          if self._price < candidate_price:
            self._symbol = stock_request.symbol
            self._price = candidate_price

    def terminate(self):
      control.control()
      if active():
        if self._symbol is None:
          raise ValueError()
        else:
          stock_reply_callback(
              stock_pb2.StockReply(symbol=self._symbol, price=self._price))
          self._symbol = None
          self._price = None

    def consume_and_terminate(self, stock_request):
      control.control()
      if active():
        if self._price is None:
          stock_reply_callback(
              stock_pb2.StockReply(
                  symbol=stock_request.symbol,
                  price=_price(stock_request.symbol)))
        else:
          candidate_price = _price(stock_request.symbol)
          if self._price < candidate_price:
            stock_reply_callback(
                stock_pb2.StockReply(
                    symbol=stock_request.symbol, price=candidate_price))
          else:
            stock_reply_callback(
                stock_pb2.StockReply(
                    symbol=self._symbol, price=self._price))

        self._symbol = None
        self._price = None

  return StockRequestConsumer()


class GetLastTradePrice(service.UnaryUnaryTestMethodImplementation):
  """GetLastTradePrice for use in tests."""

  def name(self):
    return 'GetLastTradePrice'

  def cardinality(self):
    return cardinality.Cardinality.UNARY_UNARY

  def request_class(self):
    return stock_pb2.StockRequest

  def response_class(self):
    return stock_pb2.StockReply

  def serialize_request(self, request):
    return request.SerializeToString()

  def deserialize_request(self, serialized_request):
    return stock_pb2.StockRequest.FromString(serialized_request)

  def serialize_response(self, response):
    return response.SerializeToString()

  def deserialize_response(self, serialized_response):
    return stock_pb2.StockReply.FromString(serialized_response)

  def service(self, request, response_callback, context, control):
    _get_last_trade_price(
        request, response_callback, control, context.is_active)


class GetLastTradePriceMessages(service.UnaryUnaryTestMessages):

  def __init__(self):
    self._index = 0

  def request(self):
    symbol = SYMBOL_FORMAT % self._index
    self._index += 1
    return stock_pb2.StockRequest(symbol=symbol)

  def verify(self, request, response, test_case):
    test_case.assertEqual(request.symbol, response.symbol)
    test_case.assertEqual(_price(request.symbol), response.price)


class GetLastTradePriceMultiple(service.StreamStreamTestMethodImplementation):
  """GetLastTradePriceMultiple for use in tests."""

  def name(self):
    return 'GetLastTradePriceMultiple'

  def cardinality(self):
    return cardinality.Cardinality.STREAM_STREAM

  def request_class(self):
    return stock_pb2.StockRequest

  def response_class(self):
    return stock_pb2.StockReply

  def serialize_request(self, request):
    return request.SerializeToString()

  def deserialize_request(self, serialized_request):
    return stock_pb2.StockRequest.FromString(serialized_request)

  def serialize_response(self, response):
    return response.SerializeToString()

  def deserialize_response(self, serialized_response):
    return stock_pb2.StockReply.FromString(serialized_response)

  def service(self, response_consumer, context, control):
    return _get_last_trade_price_multiple(
        response_consumer, control, context.is_active)


class GetLastTradePriceMultipleMessages(service.StreamStreamTestMessages):
  """Pairs of message streams for use with GetLastTradePriceMultiple."""

  def __init__(self):
    self._index = 0

  def requests(self):
    base_index = self._index
    self._index += 1
    return [
        stock_pb2.StockRequest(symbol=SYMBOL_FORMAT % (base_index + index))
        for index in range(STREAM_LENGTH)]

  def verify(self, requests, responses, test_case):
    test_case.assertEqual(len(requests), len(responses))
    for stock_request, stock_reply in zip(requests, responses):
      test_case.assertEqual(stock_request.symbol, stock_reply.symbol)
      test_case.assertEqual(_price(stock_request.symbol), stock_reply.price)


class WatchFutureTrades(service.UnaryStreamTestMethodImplementation):
  """WatchFutureTrades for use in tests."""

  def name(self):
    return 'WatchFutureTrades'

  def cardinality(self):
    return cardinality.Cardinality.UNARY_STREAM

  def request_class(self):
    return stock_pb2.StockRequest

  def response_class(self):
    return stock_pb2.StockReply

  def serialize_request(self, request):
    return request.SerializeToString()

  def deserialize_request(self, serialized_request):
    return stock_pb2.StockRequest.FromString(serialized_request)

  def serialize_response(self, response):
    return response.SerializeToString()

  def deserialize_response(self, serialized_response):
    return stock_pb2.StockReply.FromString(serialized_response)

  def service(self, request, response_consumer, context, control):
    _watch_future_trades(request, response_consumer, control, context.is_active)


class WatchFutureTradesMessages(service.UnaryStreamTestMessages):
  """Pairs of a single request message and a sequence of response messages."""

  def __init__(self):
    self._index = 0

  def request(self):
    symbol = SYMBOL_FORMAT % self._index
    self._index += 1
    return stock_pb2.StockRequest(
        symbol=symbol, num_trades_to_watch=STREAM_LENGTH)

  def verify(self, request, responses, test_case):
    test_case.assertEqual(STREAM_LENGTH, len(responses))
    base_price = _price(request.symbol)
    for index, response in enumerate(responses):
      test_case.assertEqual(base_price + index, response.price)


class GetHighestTradePrice(service.StreamUnaryTestMethodImplementation):
  """GetHighestTradePrice for use in tests."""

  def name(self):
    return 'GetHighestTradePrice'

  def cardinality(self):
    return cardinality.Cardinality.STREAM_UNARY

  def request_class(self):
    return stock_pb2.StockRequest

  def response_class(self):
    return stock_pb2.StockReply

  def serialize_request(self, request):
    return request.SerializeToString()

  def deserialize_request(self, serialized_request):
    return stock_pb2.StockRequest.FromString(serialized_request)

  def serialize_response(self, response):
    return response.SerializeToString()

  def deserialize_response(self, serialized_response):
    return stock_pb2.StockReply.FromString(serialized_response)

  def service(self, response_callback, context, control):
    return _get_highest_trade_price(
        response_callback, control, context.is_active)


class GetHighestTradePriceMessages(service.StreamUnaryTestMessages):

  def requests(self):
    return [
        stock_pb2.StockRequest(symbol=SYMBOL_FORMAT % index)
        for index in range(STREAM_LENGTH)]

  def verify(self, requests, response, test_case):
    price = None
    symbol = None
    for stock_request in requests:
      current_symbol = stock_request.symbol
      current_price = _price(current_symbol)
      if price is None or price < current_price:
        price = current_price
        symbol = current_symbol
    test_case.assertEqual(price, response.price)
    test_case.assertEqual(symbol, response.symbol)


class StockTestService(service.TestService):
  """A corpus of test data with one method of each RPC cardinality."""

  def name(self):
    return 'Stock'

  def unary_unary_scenarios(self):
    return {
        'GetLastTradePrice': (
            GetLastTradePrice(), [GetLastTradePriceMessages()]),
    }

  def unary_stream_scenarios(self):
    return {
        'WatchFutureTrades': (
            WatchFutureTrades(), [WatchFutureTradesMessages()]),
    }

  def stream_unary_scenarios(self):
    return {
        'GetHighestTradePrice': (
            GetHighestTradePrice(), [GetHighestTradePriceMessages()])
    }

  def stream_stream_scenarios(self):
    return {
        'GetLastTradePriceMultiple': (
            GetLastTradePriceMultiple(), [GetLastTradePriceMultipleMessages()]),
    }


STOCK_TEST_SERVICE = StockTestService()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Package-internal interfaces."""

import abc

# interfaces is referenced from specification in this module.
from grpc.framework.base import interfaces  # pylint: disable=unused-import
from grpc.framework.foundation import stream


class TerminationManager(object):
  """An object responsible for handling the termination of an operation."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def set_expiration_manager(self, expiration_manager):
    """Sets the ExpirationManager with which this object will cooperate."""
    raise NotImplementedError()

  @abc.abstractmethod
  def is_active(self):
    """Reports whether or not the operation is active.

    Returns:
      True if the operation is active or False if the operation has terminated.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def add_callback(self, callback):
    """Registers a callback to be called on operation termination.

    If the operation has already terminated, the callback will be called
    immediately.

    Args:
      callback: A callable that will be passed an interfaces.Outcome value.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def emission_complete(self):
    """Indicates that emissions from customer code have completed."""
    raise NotImplementedError()

  @abc.abstractmethod
  def transmission_complete(self):
    """Indicates that transmissions to the remote end are complete."""
    raise NotImplementedError()

  @abc.abstractmethod
  def ingestion_complete(self):
    """Indicates that customer code ingestion of received values is complete."""
    raise NotImplementedError()

  @abc.abstractmethod
  def abort(self, outcome):
    """Indicates that the operation must abort for the indicated reason.

    Args:
      outcome: An interfaces.Outcome indicating operation abortion.
    """
    raise NotImplementedError()


class TransmissionManager(object):
  """A manager responsible for transmitting to the other end of an operation."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def inmit(self, emission, complete):
    """Accepts a value for transmission to the other end of the operation.

    Args:
      emission: A value of some significance to the customer to be transmitted
        to the other end of the operation. May be None only if complete is True.
      complete: A boolean that if True indicates that customer code has emitted
        all values it intends to emit.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def abort(self, outcome):
    """Indicates that the operation has aborted for the indicated reason.

    Args:
      outcome: An interfaces.Outcome indicating operation abortion.
    """
    raise NotImplementedError()


class EmissionManager(stream.Consumer):
  """A manager of values emitted by customer code."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def set_ingestion_manager_and_expiration_manager(
      self, ingestion_manager, expiration_manager):
    """Sets two other objects with which this EmissionManager will cooperate.

    Args:
      ingestion_manager: The IngestionManager for the operation.
      expiration_manager: The ExpirationManager for the operation.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def consume(self, value):
    """Accepts a value emitted by customer code.

    This method should only be called by customer code.

    Args:
      value: Any value of significance to the customer.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def terminate(self):
    """Indicates that no more values will be emitted by customer code.

    This method should only be called by customer code.

    Implementations of this method may be idempotent and forgive customer code
    calling this method more than once.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def consume_and_terminate(self, value):
    """Accepts the last value emitted by customer code.

    This method should only be called by customer code.

    Args:
      value: Any value of significance to the customer.
    """
    raise NotImplementedError()


class IngestionManager(stream.Consumer):
  """A manager responsible for executing customer code."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def set_expiration_manager(self, expiration_manager):
    """Sets the ExpirationManager with which this object will cooperate."""
    raise NotImplementedError()

  @abc.abstractmethod
  def start(self, requirement):
    """Commences execution of customer code.

    Args:
      requirement: Some value unavailable at the time of this object's
        construction that is required to begin executing customer code.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def consume(self, payload):
    """Accepts a customer-significant value to be supplied to customer code.

    Args:
      payload: Some customer-significant value.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def terminate(self):
    """Indicates the end of values to be supplied to customer code."""
    raise NotImplementedError()

  @abc.abstractmethod
  def consume_and_terminate(self, payload):
    """Accepts the last value to be supplied to customer code.

    Args:
      payload: Some customer-significant value (and the last such value).
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def abort(self):
    """Indicates to this manager that the operation has aborted."""
    raise NotImplementedError()


class ExpirationManager(object):
  """A manager responsible for aborting the operation if it runs out of time."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def change_timeout(self, timeout):
    """Changes the timeout allotted for the operation.

    Operation duration is always measure from the beginning of the operation;
    calling this method changes the operation's allotted time to timeout total
    seconds, not timeout seconds from the time of this method call.

    Args:
      timeout: A length of time in seconds to allow for the operation.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def deadline(self):
    """Returns the time until which the operation is allowed to run.

    Returns:
      The time (seconds since the epoch) at which the operation will expire.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def abort(self):
    """Indicates to this manager that the operation has aborted."""
    raise NotImplementedError()


class ReceptionManager(object):
  """A manager responsible for receiving tickets from the other end."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def receive_ticket(self, ticket):
    """Handle a ticket from the other side of the operation.

    Args:
      ticket: An interfaces.BackToFrontTicket or interfaces.FrontToBackTicket
        appropriate to this end of the operation and this object.
    """
    raise NotImplementedError()


class CancellationManager(object):
  """A manager of operation cancellation."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def cancel(self):
    """Cancels the operation."""
    raise NotImplementedError()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Utilities for use with GRPC."""

from grpc.framework.alpha import interfaces


class _RpcMethodDescription(
    interfaces.RpcMethodInvocationDescription,
    interfaces.RpcMethodServiceDescription):

  def __init__(
      self, cardinality, unary_unary, unary_stream, stream_unary,
      stream_stream, request_serializer, request_deserializer,
      response_serializer, response_deserializer):
    self._cardinality = cardinality
    self._unary_unary = unary_unary
    self._unary_stream = unary_stream
    self._stream_unary = stream_unary
    self._stream_stream = stream_stream
    self._request_serializer = request_serializer
    self._request_deserializer = request_deserializer
    self._response_serializer = response_serializer
    self._response_deserializer = response_deserializer

  def cardinality(self):
    """See interfaces.RpcMethodDescription.cardinality for specification."""
    return self._cardinality

  def serialize_request(self, request):
    """See interfaces.RpcMethodInvocationDescription.serialize_request."""
    return self._request_serializer(request)

  def deserialize_request(self, serialized_request):
    """See interfaces.RpcMethodServiceDescription.deserialize_request."""
    return self._request_deserializer(serialized_request)

  def serialize_response(self, response):
    """See interfaces.RpcMethodServiceDescription.serialize_response."""
    return self._response_serializer(response)

  def deserialize_response(self, serialized_response):
    """See interfaces.RpcMethodInvocationDescription.deserialize_response."""
    return self._response_deserializer(serialized_response)

  def service_unary_unary(self, request, context):
    """See interfaces.RpcMethodServiceDescription.service_unary_unary."""
    return self._unary_unary(request, context)

  def service_unary_stream(self, request, context):
    """See interfaces.RpcMethodServiceDescription.service_unary_stream."""
    return self._unary_stream(request, context)

  def service_stream_unary(self, request_iterator, context):
    """See interfaces.RpcMethodServiceDescription.service_stream_unary."""
    return self._stream_unary(request_iterator, context)

  def service_stream_stream(self, request_iterator, context):
    """See interfaces.RpcMethodServiceDescription.service_stream_stream."""
    return self._stream_stream(request_iterator, context)


def unary_unary_invocation_description(
    request_serializer, response_deserializer):
  """Creates an interfaces.RpcMethodInvocationDescription for an RPC method.

  Args:
    request_serializer: A callable that when called on a request
      value returns a bytestring corresponding to that value.
    response_deserializer: A callable that when called on a
      bytestring returns the response value corresponding to
      that bytestring.

  Returns:
    An interfaces.RpcMethodInvocationDescription constructed from the given
      arguments representing a unary-request/unary-response RPC method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.UNARY_UNARY, None, None, None, None,
      request_serializer, None, None, response_deserializer)


def unary_stream_invocation_description(
    request_serializer, response_deserializer):
  """Creates an interfaces.RpcMethodInvocationDescription for an RPC method.

  Args:
    request_serializer: A callable that when called on a request
      value returns a bytestring corresponding to that value.
    response_deserializer: A callable that when called on a
      bytestring returns the response value corresponding to
      that bytestring.

  Returns:
    An interfaces.RpcMethodInvocationDescription constructed from the given
      arguments representing a unary-request/streaming-response RPC method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.UNARY_STREAM, None, None, None, None,
      request_serializer, None, None, response_deserializer)


def stream_unary_invocation_description(
    request_serializer, response_deserializer):
  """Creates an interfaces.RpcMethodInvocationDescription for an RPC method.

  Args:
    request_serializer: A callable that when called on a request
      value returns a bytestring corresponding to that value.
    response_deserializer: A callable that when called on a
      bytestring returns the response value corresponding to
      that bytestring.

  Returns:
    An interfaces.RpcMethodInvocationDescription constructed from the given
      arguments representing a streaming-request/unary-response RPC method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.STREAM_UNARY, None, None, None, None,
      request_serializer, None, None, response_deserializer)


def stream_stream_invocation_description(
    request_serializer, response_deserializer):
  """Creates an interfaces.RpcMethodInvocationDescription for an RPC method.

  Args:
    request_serializer: A callable that when called on a request
      value returns a bytestring corresponding to that value.
    response_deserializer: A callable that when called on a
      bytestring returns the response value corresponding to
      that bytestring.

  Returns:
    An interfaces.RpcMethodInvocationDescription constructed from the given
      arguments representing a  streaming-request/streaming-response RPC
      method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.STREAM_STREAM, None, None, None, None,
      request_serializer, None, None, response_deserializer)


def unary_unary_service_description(
    behavior, request_deserializer, response_serializer):
  """Creates an interfaces.RpcMethodServiceDescription for the given behavior.

  Args:
    behavior: A callable that implements a unary-unary RPC
      method that accepts a single request and an interfaces.RpcContext and
      returns a single response.
    request_deserializer: A callable that when called on a
      bytestring returns the request value corresponding to that
      bytestring.
    response_serializer: A callable that when called on a
      response value returns the bytestring corresponding to
      that value.

  Returns:
    An interfaces.RpcMethodServiceDescription constructed from the given
      arguments representing a unary-request/unary-response RPC
      method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.UNARY_UNARY, behavior, None, None, None,
      None, request_deserializer, response_serializer, None)


def unary_stream_service_description(
    behavior, request_deserializer, response_serializer):
  """Creates an interfaces.RpcMethodServiceDescription for the given behavior.

  Args:
    behavior: A callable that implements a unary-stream RPC
      method that accepts a single request and an interfaces.RpcContext
      and returns an iterator of zero or more responses.
    request_deserializer: A callable that when called on a
      bytestring returns the request value corresponding to that
      bytestring.
    response_serializer: A callable that when called on a
      response value returns the bytestring corresponding to
      that value.

  Returns:
    An interfaces.RpcMethodServiceDescription constructed from the given
      arguments representing a unary-request/streaming-response
      RPC method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.UNARY_STREAM, None, behavior, None, None,
      None, request_deserializer, response_serializer, None)


def stream_unary_service_description(
    behavior, request_deserializer, response_serializer):
  """Creates an interfaces.RpcMethodServiceDescription for the given behavior.

  Args:
    behavior: A callable that implements a stream-unary RPC
      method that accepts an iterator of zero or more requests
      and an interfaces.RpcContext and returns a single response.
    request_deserializer: A callable that when called on a
      bytestring returns the request value corresponding to that
      bytestring.
    response_serializer: A callable that when called on a
      response value returns the bytestring corresponding to
      that value.

  Returns:
    An interfaces.RpcMethodServiceDescription constructed from the given
      arguments representing a streaming-request/unary-response
      RPC method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.STREAM_UNARY, None, None, behavior, None,
      None, request_deserializer, response_serializer, None)


def stream_stream_service_description(
    behavior, request_deserializer, response_serializer):
  """Creates an interfaces.RpcMethodServiceDescription for the given behavior.

  Args:
    behavior: A callable that implements a stream-stream RPC
      method that accepts an iterator of zero or more requests
      and an interfaces.RpcContext and returns an iterator of
      zero or more responses.
    request_deserializer: A callable that when called on a
      bytestring returns the request value corresponding to that
      bytestring.
    response_serializer: A callable that when called on a
      response value returns the bytestring corresponding to
      that value.

  Returns:
    An interfaces.RpcMethodServiceDescription constructed from the given
      arguments representing a
      streaming-request/streaming-response RPC method.
  """
  return _RpcMethodDescription(
      interfaces.Cardinality.STREAM_STREAM, None, None, None, behavior,
      None, request_deserializer, response_serializer, None)



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A test to verify an implementation of the Face layer of RPC Framework."""

# unittest is referenced from specification in this module.
import abc
import unittest  # pylint: disable=unused-import

from grpc.framework.face import exceptions
from grpc.framework.face.testing import control
from grpc.framework.face.testing import coverage
from grpc.framework.face.testing import digest
from grpc.framework.face.testing import stock_service
from grpc.framework.face.testing import test_case

_TIMEOUT = 3
_LONG_TIMEOUT = 45


class BlockingInvocationInlineServiceTestCase(
    test_case.FaceTestCase, coverage.BlockingCoverage):
  """A test of the Face layer of RPC Framework.

  Concrete subclasses must also extend unittest.TestCase.
  """
  __metaclass__ = abc.ABCMeta

  def setUp(self):
    """See unittest.TestCase.setUp for full specification.

    Overriding implementations must call this implementation.
    """
    self.control = control.PauseFailControl()
    self.digest = digest.digest(
        stock_service.STOCK_TEST_SERVICE, self.control, None)

    self.stub, self.memo = self.set_up_implementation(
        self.digest.name, self.digest.methods,
        self.digest.inline_method_implementations, None)

  def tearDown(self):
    """See unittest.TestCase.tearDown for full specification.

    Overriding implementations must call this implementation.
    """
    self.tear_down_implementation(self.memo)

  def testSuccessfulUnaryRequestUnaryResponse(self):
    for name, test_messages_sequence in (
        self.digest.unary_unary_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        request = test_messages.request()

        response = self.stub.blocking_value_in_value_out(
            name, request, _LONG_TIMEOUT)

        test_messages.verify(request, response, self)

  def testSuccessfulUnaryRequestStreamResponse(self):
    for name, test_messages_sequence in (
        self.digest.unary_stream_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        request = test_messages.request()

        response_iterator = self.stub.inline_value_in_stream_out(
            name, request, _LONG_TIMEOUT)
        responses = list(response_iterator)

        test_messages.verify(request, responses, self)

  def testSuccessfulStreamRequestUnaryResponse(self):
    for name, test_messages_sequence in (
        self.digest.stream_unary_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        requests = test_messages.requests()

        response = self.stub.blocking_stream_in_value_out(
            name, iter(requests), _LONG_TIMEOUT)

        test_messages.verify(requests, response, self)

  def testSuccessfulStreamRequestStreamResponse(self):
    for name, test_messages_sequence in (
        self.digest.stream_stream_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        requests = test_messages.requests()

        response_iterator = self.stub.inline_stream_in_stream_out(
            name, iter(requests), _LONG_TIMEOUT)
        responses = list(response_iterator)

        test_messages.verify(requests, responses, self)

  def testSequentialInvocations(self):
    for name, test_messages_sequence in (
        self.digest.unary_unary_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        first_request = test_messages.request()
        second_request = test_messages.request()

        first_response = self.stub.blocking_value_in_value_out(
            name, first_request, _TIMEOUT)

        test_messages.verify(first_request, first_response, self)

        second_response = self.stub.blocking_value_in_value_out(
            name, second_request, _TIMEOUT)

        test_messages.verify(second_request, second_response, self)

  def testExpiredUnaryRequestUnaryResponse(self):
    for name, test_messages_sequence in (
        self.digest.unary_unary_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        request = test_messages.request()

        with self.control.pause(), self.assertRaises(
            exceptions.ExpirationError):
          multi_callable = self.stub.unary_unary_multi_callable(name)
          multi_callable(request, _TIMEOUT)

  def testExpiredUnaryRequestStreamResponse(self):
    for name, test_messages_sequence in (
        self.digest.unary_stream_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        request = test_messages.request()

        with self.control.pause(), self.assertRaises(
            exceptions.ExpirationError):
          response_iterator = self.stub.inline_value_in_stream_out(
              name, request, _TIMEOUT)
          list(response_iterator)

  def testExpiredStreamRequestUnaryResponse(self):
    for name, test_messages_sequence in (
        self.digest.stream_unary_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        requests = test_messages.requests()

        with self.control.pause(), self.assertRaises(
            exceptions.ExpirationError):
          multi_callable = self.stub.stream_unary_multi_callable(name)
          multi_callable(iter(requests), _TIMEOUT)

  def testExpiredStreamRequestStreamResponse(self):
    for name, test_messages_sequence in (
        self.digest.stream_stream_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        requests = test_messages.requests()

        with self.control.pause(), self.assertRaises(
            exceptions.ExpirationError):
          response_iterator = self.stub.inline_stream_in_stream_out(
              name, iter(requests), _TIMEOUT)
          list(response_iterator)

  def testFailedUnaryRequestUnaryResponse(self):
    for name, test_messages_sequence in (
        self.digest.unary_unary_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        request = test_messages.request()

        with self.control.fail(), self.assertRaises(exceptions.ServicerError):
          self.stub.blocking_value_in_value_out(name, request, _TIMEOUT)

  def testFailedUnaryRequestStreamResponse(self):
    for name, test_messages_sequence in (
        self.digest.unary_stream_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        request = test_messages.request()

        with self.control.fail(), self.assertRaises(exceptions.ServicerError):
          response_iterator = self.stub.inline_value_in_stream_out(
              name, request, _TIMEOUT)
          list(response_iterator)

  def testFailedStreamRequestUnaryResponse(self):
    for name, test_messages_sequence in (
        self.digest.stream_unary_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        requests = test_messages.requests()

        with self.control.fail(), self.assertRaises(exceptions.ServicerError):
          self.stub.blocking_stream_in_value_out(name, iter(requests), _TIMEOUT)

  def testFailedStreamRequestStreamResponse(self):
    for name, test_messages_sequence in (
        self.digest.stream_stream_messages_sequences.iteritems()):
      for test_messages in test_messages_sequence:
        requests = test_messages.requests()

        with self.control.fail(), self.assertRaises(exceptions.ServicerError):
          response_iterator = self.stub.inline_stream_in_stream_out(
              name, iter(requests), _TIMEOUT)
          list(response_iterator)



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Common construction and destruction for GRPC-backed Face-layer tests."""

import unittest

from grpc._adapter import fore
from grpc._adapter import rear
from grpc.framework.base import util
from grpc.framework.base import implementations as base_implementations
from grpc.framework.face import implementations as face_implementations
from grpc.framework.face.testing import coverage
from grpc.framework.face.testing import serial
from grpc.framework.face.testing import test_case
from grpc.framework.foundation import logging_pool

_TIMEOUT = 3
_MAXIMUM_TIMEOUT = 90
_MAXIMUM_POOL_SIZE = 400


class FaceTestCase(test_case.FaceTestCase, coverage.BlockingCoverage):
  """Provides abstract Face-layer tests a GRPC-backed implementation."""

  def set_up_implementation(
      self, name, methods, method_implementations,
      multi_method_implementation):
    pool = logging_pool.pool(_MAXIMUM_POOL_SIZE)

    servicer = face_implementations.servicer(
        pool, method_implementations, multi_method_implementation)

    serialization = serial.serialization(methods)

    fore_link = fore.ForeLink(
        pool, serialization.request_deserializers,
        serialization.response_serializers, None, ())
    fore_link.start()
    port = fore_link.port()
    rear_link = rear.RearLink(
        'localhost', port, pool,
        serialization.request_serializers,
        serialization.response_deserializers, False, None, None, None)
    rear_link.start()
    front = base_implementations.front_link(pool, pool, pool)
    back = base_implementations.back_link(
        servicer, pool, pool, pool, _TIMEOUT, _MAXIMUM_TIMEOUT)
    fore_link.join_rear_link(back)
    back.join_fore_link(fore_link)
    rear_link.join_fore_link(front)
    front.join_rear_link(rear_link)

    stub = face_implementations.generic_stub(front, pool)
    return stub, (rear_link, fore_link, front, back)

  def tear_down_implementation(self, memo):
    rear_link, fore_link, front, back = memo
    # TODO(nathaniel): Waiting for the front and back to idle possibly should
    # not be necessary - investigate as part of graceful shutdown work.
    util.wait_for_idle(front)
    util.wait_for_idle(back)
    rear_link.stop()
    fore_link.stop()

  @unittest.skip('Service-side failure not transmitted by GRPC.')
  def testFailedUnaryRequestUnaryResponse(self):
    raise NotImplementedError()

  @unittest.skip('Service-side failure not transmitted by GRPC.')
  def testFailedUnaryRequestStreamResponse(self):
    raise NotImplementedError()

  @unittest.skip('Service-side failure not transmitted by GRPC.')
  def testFailedStreamRequestUnaryResponse(self):
    raise NotImplementedError()

  @unittest.skip('Service-side failure not transmitted by GRPC.')
  def testFailedStreamRequestStreamResponse(self):
    raise NotImplementedError()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Helpful utilities related to the stream module."""

import logging
import threading

from grpc.framework.foundation import stream

_NO_VALUE = object()


class TransformingConsumer(stream.Consumer):
  """A stream.Consumer that passes a transformation of its input to another."""

  def __init__(self, transformation, downstream):
    self._transformation = transformation
    self._downstream = downstream

  def consume(self, value):
    self._downstream.consume(self._transformation(value))

  def terminate(self):
    self._downstream.terminate()

  def consume_and_terminate(self, value):
    self._downstream.consume_and_terminate(self._transformation(value))


class IterableConsumer(stream.Consumer):
  """A Consumer that when iterated over emits the values it has consumed."""

  def __init__(self):
    self._condition = threading.Condition()
    self._values = []
    self._active = True

  def consume(self, stock_reply):
    with self._condition:
      if self._active:
        self._values.append(stock_reply)
        self._condition.notify()

  def terminate(self):
    with self._condition:
      self._active = False
      self._condition.notify()

  def consume_and_terminate(self, stock_reply):
    with self._condition:
      if self._active:
        self._values.append(stock_reply)
        self._active = False
        self._condition.notify()

  def __iter__(self):
    return self

  def next(self):
    with self._condition:
      while self._active and not self._values:
        self._condition.wait()
      if self._values:
        return self._values.pop(0)
      else:
        raise StopIteration()


class ThreadSwitchingConsumer(stream.Consumer):
  """A Consumer decorator that affords serialization and asynchrony."""

  def __init__(self, sink, pool):
    self._lock = threading.Lock()
    self._sink = sink
    self._pool = pool
    # True if self._spin has been submitted to the pool to be called once and
    # that call has not yet returned, False otherwise.
    self._spinning = False
    self._values = []
    self._active = True

  def _spin(self, sink, value, terminate):
    while True:
      try:
        if value is _NO_VALUE:
          sink.terminate()
        elif terminate:
          sink.consume_and_terminate(value)
        else:
          sink.consume(value)
      except Exception as e:  # pylint:disable=broad-except
        logging.exception(e)

      with self._lock:
        if terminate:
          self._spinning = False
          return
        elif self._values:
          value = self._values.pop(0)
          terminate = not self._values and not self._active
        elif not self._active:
          value = _NO_VALUE
          terminate = True
        else:
          self._spinning = False
          return

  def consume(self, value):
    with self._lock:
      if self._active:
        if self._spinning:
          self._values.append(value)
        else:
          self._pool.submit(self._spin, self._sink, value, False)
          self._spinning = True

  def terminate(self):
    with self._lock:
      if self._active:
        self._active = False
        if not self._spinning:
          self._pool.submit(self._spin, self._sink, _NO_VALUE, True)
          self._spinning = True

  def consume_and_terminate(self, value):
    with self._lock:
      if self._active:
        self._active = False
        if self._spinning:
          self._values.append(value)
        else:
          self._pool.submit(self._spin, self._sink, value, True)
          self._spinning = True



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Tools for creating tests of implementations of the Face layer."""

import abc

# face_interfaces and interfaces are referenced in specification in this module.
from grpc.framework.face import interfaces as face_interfaces  # pylint: disable=unused-import
from grpc.framework.face.testing import interfaces  # pylint: disable=unused-import


class FaceTestCase(object):
  """Describes a test of the Face Layer of RPC Framework.

  Concrete subclasses must also inherit from unittest.TestCase and from at least
  one class that defines test methods.
  """
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def set_up_implementation(
      self, name, methods, method_implementations,
      multi_method_implementation):
    """Instantiates the Face Layer implementation under test.

    Args:
      name: The service name to be used in the test.
      methods: A sequence of interfaces.Method objects describing the RPC
        methods that will be called during the test.
      method_implementations: A dictionary from string RPC method name to
        face_interfaces.MethodImplementation object specifying
        implementation of an RPC method.
      multi_method_implementation: An face_interfaces.MultiMethodImplementation
        or None.

    Returns:
      A sequence of length two the first element of which is a
        face_interfaces.GenericStub (backed by the given method
        implementations), and the second element of which is an arbitrary memo
        object to be kept and passed to tearDownImplementation at the conclusion
        of the test.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def tear_down_implementation(self, memo):
    """Destroys the Face layer implementation under test.

    Args:
      memo: The object from the second position of the return value of
        set_up_implementation.
    """
    raise NotImplementedError()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""State and behavior for ticket transmission during an operation."""

import abc

from grpc.framework.base import _constants
from grpc.framework.base import _interfaces
from grpc.framework.base import interfaces
from grpc.framework.foundation import callable_util

_TRANSMISSION_EXCEPTION_LOG_MESSAGE = 'Exception during transmission!'

_FRONT_TO_BACK_NO_TRANSMISSION_OUTCOMES = (
    interfaces.Outcome.SERVICER_FAILURE,
    )
_BACK_TO_FRONT_NO_TRANSMISSION_OUTCOMES = (
    interfaces.Outcome.CANCELLED,
    interfaces.Outcome.SERVICED_FAILURE,
    )

_ABORTION_OUTCOME_TO_FRONT_TO_BACK_TICKET_KIND = {
    interfaces.Outcome.CANCELLED:
        interfaces.FrontToBackTicket.Kind.CANCELLATION,
    interfaces.Outcome.EXPIRED:
        interfaces.FrontToBackTicket.Kind.EXPIRATION,
    interfaces.Outcome.RECEPTION_FAILURE:
        interfaces.FrontToBackTicket.Kind.RECEPTION_FAILURE,
    interfaces.Outcome.TRANSMISSION_FAILURE:
        interfaces.FrontToBackTicket.Kind.TRANSMISSION_FAILURE,
    interfaces.Outcome.SERVICED_FAILURE:
        interfaces.FrontToBackTicket.Kind.SERVICED_FAILURE,
    interfaces.Outcome.SERVICER_FAILURE:
        interfaces.FrontToBackTicket.Kind.SERVICER_FAILURE,
}

_ABORTION_OUTCOME_TO_BACK_TO_FRONT_TICKET_KIND = {
    interfaces.Outcome.CANCELLED:
        interfaces.BackToFrontTicket.Kind.CANCELLATION,
    interfaces.Outcome.EXPIRED:
        interfaces.BackToFrontTicket.Kind.EXPIRATION,
    interfaces.Outcome.RECEPTION_FAILURE:
        interfaces.BackToFrontTicket.Kind.RECEPTION_FAILURE,
    interfaces.Outcome.TRANSMISSION_FAILURE:
        interfaces.BackToFrontTicket.Kind.TRANSMISSION_FAILURE,
    interfaces.Outcome.SERVICED_FAILURE:
        interfaces.BackToFrontTicket.Kind.SERVICED_FAILURE,
    interfaces.Outcome.SERVICER_FAILURE:
        interfaces.BackToFrontTicket.Kind.SERVICER_FAILURE,
}


class _Ticketizer(object):
  """Common specification of different ticket-creating behavior."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def ticketize(self, operation_id, sequence_number, payload, complete):
    """Creates a ticket indicating ordinary operation progress.

    Args:
      operation_id: The operation ID for the current operation.
      sequence_number: A sequence number for the ticket.
      payload: A customer payload object. May be None if sequence_number is
        zero or complete is true.
      complete: A boolean indicating whether or not the ticket should describe
        itself as (but for a later indication of operation abortion) the last
        ticket to be sent.

    Returns:
      An object of an appropriate type suitable for transmission to the other
        side of the operation.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def ticketize_abortion(self, operation_id, sequence_number, outcome):
    """Creates a ticket indicating that the operation is aborted.

    Args:
      operation_id: The operation ID for the current operation.
      sequence_number: A sequence number for the ticket.
      outcome: An interfaces.Outcome value describing the operation abortion.

    Returns:
      An object of an appropriate type suitable for transmission to the other
        side of the operation, or None if transmission is not appropriate for
        the given outcome.
    """
    raise NotImplementedError()


class _FrontTicketizer(_Ticketizer):
  """Front-side ticket-creating behavior."""

  def __init__(self, name, subscription_kind, trace_id, timeout):
    """Constructor.

    Args:
      name: The name of the operation.
      subscription_kind: An interfaces.ServicedSubscription.Kind value
        describing the interest the front has in tickets sent from the back.
      trace_id: A uuid.UUID identifying a set of related operations to which
        this operation belongs.
      timeout: A length of time in seconds to allow for the entire operation.
    """
    self._name = name
    self._subscription_kind = subscription_kind
    self._trace_id = trace_id
    self._timeout = timeout

  def ticketize(self, operation_id, sequence_number, payload, complete):
    """See _Ticketizer.ticketize for specification."""
    if sequence_number:
      if complete:
        kind = interfaces.FrontToBackTicket.Kind.COMPLETION
      else:
        kind = interfaces.FrontToBackTicket.Kind.CONTINUATION
      return interfaces.FrontToBackTicket(
          operation_id, sequence_number, kind, self._name,
          self._subscription_kind, self._trace_id, payload, self._timeout)
    else:
      if complete:
        kind = interfaces.FrontToBackTicket.Kind.ENTIRE
      else:
        kind = interfaces.FrontToBackTicket.Kind.COMMENCEMENT
      return interfaces.FrontToBackTicket(
          operation_id, 0, kind, self._name, self._subscription_kind,
          self._trace_id, payload, self._timeout)

  def ticketize_abortion(self, operation_id, sequence_number, outcome):
    """See _Ticketizer.ticketize_abortion for specification."""
    if outcome in _FRONT_TO_BACK_NO_TRANSMISSION_OUTCOMES:
      return None
    else:
      kind = _ABORTION_OUTCOME_TO_FRONT_TO_BACK_TICKET_KIND[outcome]
      return interfaces.FrontToBackTicket(
          operation_id, sequence_number, kind, None, None, None, None, None)


class _BackTicketizer(_Ticketizer):
  """Back-side ticket-creating behavior."""

  def ticketize(self, operation_id, sequence_number, payload, complete):
    """See _Ticketizer.ticketize for specification."""
    if complete:
      kind = interfaces.BackToFrontTicket.Kind.COMPLETION
    else:
      kind = interfaces.BackToFrontTicket.Kind.CONTINUATION
    return interfaces.BackToFrontTicket(
        operation_id, sequence_number, kind, payload)

  def ticketize_abortion(self, operation_id, sequence_number, outcome):
    """See _Ticketizer.ticketize_abortion for specification."""
    if outcome in _BACK_TO_FRONT_NO_TRANSMISSION_OUTCOMES:
      return None
    else:
      kind = _ABORTION_OUTCOME_TO_BACK_TO_FRONT_TICKET_KIND[outcome]
      return interfaces.BackToFrontTicket(
          operation_id, sequence_number, kind, None)


class TransmissionManager(_interfaces.TransmissionManager):
  """A _interfaces.TransmissionManager on which other managers may be set."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def set_ingestion_and_expiration_managers(
      self, ingestion_manager, expiration_manager):
    """Sets two of the other managers with which this manager may interact.

    Args:
      ingestion_manager: The _interfaces.IngestionManager associated with the
        current operation.
      expiration_manager: The _interfaces.ExpirationManager associated with the
        current operation.
    """
    raise NotImplementedError()


class _EmptyTransmissionManager(TransmissionManager):
  """A completely no-operative _interfaces.TransmissionManager."""

  def set_ingestion_and_expiration_managers(
      self, ingestion_manager, expiration_manager):
    """See overriden method for specification."""

  def inmit(self, emission, complete):
    """See _interfaces.TransmissionManager.inmit for specification."""

  def abort(self, outcome):
    """See _interfaces.TransmissionManager.abort for specification."""


class _TransmittingTransmissionManager(TransmissionManager):
  """A TransmissionManager implementation that sends tickets."""

  def __init__(
      self, lock, pool, callback, operation_id, ticketizer,
      termination_manager):
    """Constructor.

    Args:
      lock: The operation-servicing-wide lock object.
      pool: A thread pool in which the work of transmitting tickets will be
        performed.
      callback: A callable that accepts tickets and sends them to the other side
        of the operation.
      operation_id: The operation's ID.
      ticketizer: A _Ticketizer for ticket creation.
      termination_manager: The _interfaces.TerminationManager associated with
        this operation.
    """
    self._lock = lock
    self._pool = pool
    self._callback = callback
    self._operation_id = operation_id
    self._ticketizer = ticketizer
    self._termination_manager = termination_manager
    self._ingestion_manager = None
    self._expiration_manager = None

    self._emissions = []
    self._emission_complete = False
    self._outcome = None
    self._lowest_unused_sequence_number = 0
    self._transmitting = False

  def set_ingestion_and_expiration_managers(
      self, ingestion_manager, expiration_manager):
    """See overridden method for specification."""
    self._ingestion_manager = ingestion_manager
    self._expiration_manager = expiration_manager

  def _lead_ticket(self, emission, complete):
    """Creates a ticket suitable for leading off the transmission loop.

    Args:
      emission: A customer payload object to be sent to the other side of the
        operation.
      complete: Whether or not the sequence of customer payloads ends with
        the passed object.

    Returns:
      A ticket with which to lead off the transmission loop.
    """
    sequence_number = self._lowest_unused_sequence_number
    self._lowest_unused_sequence_number += 1
    return self._ticketizer.ticketize(
        self._operation_id, sequence_number, emission, complete)

  def _abortive_response_ticket(self, outcome):
    """Creates a ticket indicating operation abortion.

    Args:
      outcome: An interfaces.Outcome value describing operation abortion.

    Returns:
      A ticket indicating operation abortion.
    """
    ticket = self._ticketizer.ticketize_abortion(
        self._operation_id, self._lowest_unused_sequence_number, outcome)
    if ticket is None:
      return None
    else:
      self._lowest_unused_sequence_number += 1
      return ticket

  def _next_ticket(self):
    """Creates the next ticket to be sent to the other side of the operation.

    Returns:
      A (completed, ticket) tuple comprised of a boolean indicating whether or
        not the sequence of tickets has completed normally and a ticket to send
        to the other side if the sequence of tickets hasn't completed. The tuple
        will never have both a True first element and a non-None second element.
    """
    if self._emissions is None:
      return False, None
    elif self._outcome is None:
      if self._emissions:
        payload = self._emissions.pop(0)
        complete = self._emission_complete and not self._emissions
        sequence_number = self._lowest_unused_sequence_number
        self._lowest_unused_sequence_number += 1
        return complete, self._ticketizer.ticketize(
            self._operation_id, sequence_number, payload, complete)
      else:
        return self._emission_complete, None
    else:
      ticket = self._abortive_response_ticket(self._outcome)
      self._emissions = None
      return False, None if ticket is None else ticket

  def _transmit(self, ticket):
    """Commences the transmission loop sending tickets.

    Args:
      ticket: A ticket to be sent to the other side of the operation.
    """
    def transmit(ticket):
      while True:
        transmission_outcome = callable_util.call_logging_exceptions(
            self._callback, _TRANSMISSION_EXCEPTION_LOG_MESSAGE, ticket)
        if transmission_outcome.exception is None:
          with self._lock:
            complete, ticket = self._next_ticket()
            if ticket is None:
              if complete:
                self._termination_manager.transmission_complete()
              self._transmitting = False
              return
        else:
          with self._lock:
            self._emissions = None
            self._termination_manager.abort(
                interfaces.Outcome.TRANSMISSION_FAILURE)
            self._ingestion_manager.abort()
            self._expiration_manager.abort()
            self._transmitting = False
            return

    self._pool.submit(callable_util.with_exceptions_logged(
        transmit, _constants.INTERNAL_ERROR_LOG_MESSAGE), ticket)
    self._transmitting = True

  def inmit(self, emission, complete):
    """See _interfaces.TransmissionManager.inmit for specification."""
    if self._emissions is not None and self._outcome is None:
      self._emission_complete = complete
      if self._transmitting:
        self._emissions.append(emission)
      else:
        self._transmit(self._lead_ticket(emission, complete))

  def abort(self, outcome):
    """See _interfaces.TransmissionManager.abort for specification."""
    if self._emissions is not None and self._outcome is None:
      self._outcome = outcome
      if not self._transmitting:
        ticket = self._abortive_response_ticket(outcome)
        self._emissions = None
        if ticket is not None:
          self._transmit(ticket)


def front_transmission_manager(
    lock, pool, callback, operation_id, name, subscription_kind, trace_id,
    timeout, termination_manager):
  """Creates a TransmissionManager appropriate for front-side use.

  Args:
    lock: The operation-servicing-wide lock object.
    pool: A thread pool in which the work of transmitting tickets will be
      performed.
    callback: A callable that accepts tickets and sends them to the other side
      of the operation.
    operation_id: The operation's ID.
    name: The name of the operation.
    subscription_kind: An interfaces.ServicedSubscription.Kind value
      describing the interest the front has in tickets sent from the back.
    trace_id: A uuid.UUID identifying a set of related operations to which
      this operation belongs.
    timeout: A length of time in seconds to allow for the entire operation.
    termination_manager: The _interfaces.TerminationManager associated with
      this operation.

  Returns:
    A TransmissionManager appropriate for front-side use.
  """
  return _TransmittingTransmissionManager(
      lock, pool, callback, operation_id, _FrontTicketizer(
          name, subscription_kind, trace_id, timeout),
      termination_manager)


def back_transmission_manager(
    lock, pool, callback, operation_id, termination_manager,
    subscription_kind):
  """Creates a TransmissionManager appropriate for back-side use.

  Args:
    lock: The operation-servicing-wide lock object.
    pool: A thread pool in which the work of transmitting tickets will be
      performed.
    callback: A callable that accepts tickets and sends them to the other side
      of the operation.
    operation_id: The operation's ID.
    termination_manager: The _interfaces.TerminationManager associated with
      this operation.
    subscription_kind: An interfaces.ServicedSubscription.Kind value
      describing the interest the front has in tickets sent from the back.

  Returns:
    A TransmissionManager appropriate for back-side use.
  """
  if subscription_kind is interfaces.ServicedSubscription.Kind.NONE:
    return _EmptyTransmissionManager()
  else:
    return _TransmittingTransmissionManager(
        lock, pool, callback, operation_id, _BackTicketizer(),
        termination_manager)



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""In-memory implementations of base layer interfaces."""

import threading

from grpc.framework.base import _constants
from grpc.framework.base import interfaces
from grpc.framework.foundation import callable_util


class _Serializer(object):
  """A utility for serializing values that may arrive concurrently."""

  def __init__(self, pool):
    self._lock = threading.Lock()
    self._pool = pool
    self._sink = None
    self._spinning = False
    self._values = []

  def _spin(self, sink, value):
    while True:
      sink(value)
      with self._lock:
        if self._sink is None or not self._values:
          self._spinning = False
          return
        else:
          sink, value = self._sink, self._values.pop(0)

  def set_sink(self, sink):
    with self._lock:
      self._sink = sink
      if sink is not None and self._values and not self._spinning:
        self._spinning = True
        self._pool.submit(
            callable_util.with_exceptions_logged(
                self._spin, _constants.INTERNAL_ERROR_LOG_MESSAGE),
            sink, self._values.pop(0))

  def add_value(self, value):
    with self._lock:
      if self._sink and not self._spinning:
        self._spinning = True
        self._pool.submit(
            callable_util.with_exceptions_logged(
                self._spin, _constants.INTERNAL_ERROR_LOG_MESSAGE),
            self._sink, value)
      else:
        self._values.append(value)


class Link(interfaces.ForeLink, interfaces.RearLink):
  """A trivial implementation of interfaces.ForeLink and interfaces.RearLink."""

  def __init__(self, pool):
    """Constructor.

    Args:
      pool: A thread pool to be used for serializing ticket exchange in each
        direction.
    """
    self._front_to_back = _Serializer(pool)
    self._back_to_front = _Serializer(pool)

  def join_fore_link(self, fore_link):
    """See interfaces.RearLink.join_fore_link for specification."""
    self._back_to_front.set_sink(fore_link.accept_back_to_front_ticket)

  def join_rear_link(self, rear_link):
    """See interfaces.ForeLink.join_rear_link for specification."""
    self._front_to_back.set_sink(rear_link.accept_front_to_back_ticket)

  def accept_front_to_back_ticket(self, ticket):
    """See interfaces.ForeLink.accept_front_to_back_ticket for specification."""
    self._front_to_back.add_value(ticket)

  def accept_back_to_front_ticket(self, ticket):
    """See interfaces.RearLink.accept_back_to_front_ticket for specification."""
    self._back_to_front.add_value(ticket)



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""State and behavior for operation context."""

import time

# _interfaces is referenced from specification in this module.
from grpc.framework.base import interfaces
from grpc.framework.base import _interfaces  # pylint: disable=unused-import


class OperationContext(interfaces.OperationContext):
  """An implementation of interfaces.OperationContext."""

  def __init__(
      self, lock, operation_id, local_failure, termination_manager,
      transmission_manager):
    """Constructor.

    Args:
      lock: The operation-wide lock.
      operation_id: An object identifying the operation.
      local_failure: Whichever one of interfaces.Outcome.SERVICED_FAILURE or
        interfaces.Outcome.SERVICER_FAILURE describes local failure of
        customer code.
      termination_manager: The _interfaces.TerminationManager for the operation.
      transmission_manager: The _interfaces.TransmissionManager for the
        operation.
    """
    self._lock = lock
    self._local_failure = local_failure
    self._termination_manager = termination_manager
    self._transmission_manager = transmission_manager
    self._ingestion_manager = None
    self._expiration_manager = None

    self.operation_id = operation_id

  def set_ingestion_and_expiration_managers(
      self, ingestion_manager, expiration_manager):
    """Sets managers with which this OperationContext cooperates.

    Args:
      ingestion_manager: The _interfaces.IngestionManager for the operation.
      expiration_manager: The _interfaces.ExpirationManager for the operation.
    """
    self._ingestion_manager = ingestion_manager
    self._expiration_manager = expiration_manager

  def is_active(self):
    """See interfaces.OperationContext.is_active for specification."""
    with self._lock:
      return self._termination_manager.is_active()

  def add_termination_callback(self, callback):
    """See interfaces.OperationContext.add_termination_callback."""
    with self._lock:
      self._termination_manager.add_callback(callback)

  def time_remaining(self):
    """See interfaces.OperationContext.time_remaining for specification."""
    with self._lock:
      deadline = self._expiration_manager.deadline()
    return max(0.0, deadline - time.time())

  def fail(self, exception):
    """See interfaces.OperationContext.fail for specification."""
    with self._lock:
      self._termination_manager.abort(self._local_failure)
      self._transmission_manager.abort(self._local_failure)
      self._ingestion_manager.abort()
      self._expiration_manager.abort()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""State and behavior for operation cancellation."""

from grpc.framework.base import _interfaces
from grpc.framework.base import interfaces


class CancellationManager(_interfaces.CancellationManager):
  """An implementation of _interfaces.CancellationManager."""

  def __init__(
      self, lock, termination_manager, transmission_manager, ingestion_manager,
      expiration_manager):
    """Constructor.

    Args:
      lock: The operation-wide lock.
      termination_manager: The _interfaces.TerminationManager for the operation.
      transmission_manager: The _interfaces.TransmissionManager for the
        operation.
      ingestion_manager: The _interfaces.IngestionManager for the operation.
      expiration_manager: The _interfaces.ExpirationManager for the operation.
    """
    self._lock = lock
    self._termination_manager = termination_manager
    self._transmission_manager = transmission_manager
    self._ingestion_manager = ingestion_manager
    self._expiration_manager = expiration_manager

  def cancel(self):
    """See _interfaces.CancellationManager.cancel for specification."""
    with self._lock:
      self._termination_manager.abort(interfaces.Outcome.CANCELLED)
      self._transmission_manager.abort(interfaces.Outcome.CANCELLED)
      self._ingestion_manager.abort()
      self._expiration_manager.abort()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Tests for _adapter._c."""

import threading
import time
import unittest

from grpc._adapter import _c
from grpc._adapter import _datatypes

_TIMEOUT = 3
_FUTURE = time.time() + 60 * 60 * 24
_IDEMPOTENCE_DEMONSTRATION = 7


class _CTest(unittest.TestCase):

  def testUpAndDown(self):
    _c.init()
    _c.shut_down()

  def testCompletionQueue(self):
    _c.init()

    completion_queue = _c.CompletionQueue()
    event = completion_queue.get(0)
    self.assertIsNone(event)
    event = completion_queue.get(time.time())
    self.assertIsNone(event)
    event = completion_queue.get(time.time() + _TIMEOUT)
    self.assertIsNone(event)
    completion_queue.stop()
    for _ in range(_IDEMPOTENCE_DEMONSTRATION):
      event = completion_queue.get(time.time() + _TIMEOUT)
      self.assertIs(event.kind, _datatypes.Event.Kind.STOP)

    del completion_queue
    del event

    _c.shut_down()

  def testChannel(self):
    _c.init()

    channel = _c.Channel(
        'test host:12345', None, server_host_override='ignored')
    del channel

    _c.shut_down()

  def testCall(self):
    method = 'test method'
    host = 'test host'

    _c.init()

    channel = _c.Channel('%s:%d' % (host, 12345), None)
    call = _c.Call(channel, method, host, time.time() + _TIMEOUT)
    del call
    del channel

    _c.shut_down()

  def testServer(self):
    _c.init()

    completion_queue = _c.CompletionQueue()
    server = _c.Server(completion_queue)
    server.add_http2_addr('[::]:0')
    server.start()
    server.stop()
    completion_queue.stop()
    del server
    del completion_queue

    service_tag = object()
    completion_queue = _c.CompletionQueue()
    server = _c.Server(completion_queue)
    server.add_http2_addr('[::]:0')
    server.start()
    server.service(service_tag)
    server.stop()
    completion_queue.stop()
    event = completion_queue.get(time.time() + _TIMEOUT)
    self.assertIs(event.kind, _datatypes.Event.Kind.SERVICE_ACCEPTED)
    self.assertIs(event.tag, service_tag)
    self.assertIsNone(event.service_acceptance)
    for _ in range(_IDEMPOTENCE_DEMONSTRATION):
      event = completion_queue.get(time.time() + _TIMEOUT)
      self.assertIs(event.kind, _datatypes.Event.Kind.STOP)
    del server
    del completion_queue

    completion_queue = _c.CompletionQueue()
    server = _c.Server(completion_queue)
    server.add_http2_addr('[::]:0')
    server.start()
    thread = threading.Thread(target=completion_queue.get, args=(_FUTURE,))
    thread.start()
    time.sleep(1)
    server.stop()
    completion_queue.stop()
    for _ in range(_IDEMPOTENCE_DEMONSTRATION):
      event = completion_queue.get(time.time() + _TIMEOUT)
      self.assertIs(event.kind, _datatypes.Event.Kind.STOP)
    thread.join()
    del server
    del completion_queue

    _c.shut_down()

  def test_client_credentials(self):
    root_certificates = b'Trust starts here. Really.'
    private_key = b'This is a really bad private key, yo.'
    certificate_chain = b'Trust me! Do I not look trustworty?'

    _c.init()

    client_credentials = _c.ClientCredentials(
        None, None, None)
    self.assertIsNotNone(client_credentials)
    client_credentials = _c.ClientCredentials(
        root_certificates, None, None)
    self.assertIsNotNone(client_credentials)
    client_credentials = _c.ClientCredentials(
        None, private_key, certificate_chain)
    self.assertIsNotNone(client_credentials)
    client_credentials = _c.ClientCredentials(
        root_certificates, private_key, certificate_chain)
    self.assertIsNotNone(client_credentials)
    del client_credentials

    _c.shut_down()

  def test_server_credentials(self):
    root_certificates = b'Trust starts here. Really.'
    first_private_key = b'This is a really bad private key, yo.'
    first_certificate_chain = b'Trust me! Do I not look trustworty?'
    second_private_key = b'This is another bad private key, yo.'
    second_certificate_chain = b'Look into my eyes; you can totes trust me.'

    _c.init()

    server_credentials = _c.ServerCredentials(
        None, ((first_private_key, first_certificate_chain),))
    del server_credentials
    server_credentials = _c.ServerCredentials(
        root_certificates, ((first_private_key, first_certificate_chain),))
    del server_credentials
    server_credentials = _c.ServerCredentials(
        root_certificates,
        ((first_private_key, first_certificate_chain),
         (second_private_key, second_certificate_chain),))
    del server_credentials
    with self.assertRaises(TypeError):
      _c.ServerCredentials(
          root_certificates, first_private_key, second_certificate_chain)

    _c.shut_down()

  @unittest.skip('TODO(nathaniel): find and use real-enough test credentials')
  def test_secure_server(self):
    _c.init()

    server_credentials = _c.ServerCredentials(
        'root certificate', (('private key', 'certificate chain'),))

    completion_queue = _c.CompletionQueue()
    server = _c.Server(completion_queue, server_credentials)
    server.add_http2_addr('[::]:0')
    server.start()
    thread = threading.Thread(target=completion_queue.get, args=(_FUTURE,))
    thread.start()
    time.sleep(1)
    server.stop()
    completion_queue.stop()
    for _ in range(_IDEMPOTENCE_DEMONSTRATION):
      event = completion_queue.get(time.time() + _TIMEOUT)
      self.assertIs(event.kind, _datatypes.Event.Kind.STOP)
    thread.join()
    del server
    del completion_queue

    _c.shut_down()


if __name__ == '__main__':
  unittest.main()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Test scenarios using protocol buffers."""

import abc
import threading

from grpc._junkdrawer import math_pb2


class ProtoScenario(object):
  """An RPC test scenario using protocol buffers."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def method(self):
    """Access the test method name.

    Returns:
      The test method name.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def serialize_request(self, request):
    """Serialize a request protocol buffer.

    Args:
      request: A request protocol buffer.

    Returns:
      The bytestring serialization of the given request protocol buffer.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def deserialize_request(self, request_bytestring):
    """Deserialize a request protocol buffer.

    Args:
      request_bytestring: The bytestring serialization of a request protocol
        buffer.

    Returns:
      The request protocol buffer deserialized from the given byte string.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def serialize_response(self, response):
    """Serialize a response protocol buffer.

    Args:
      response: A response protocol buffer.

    Returns:
      The bytestring serialization of the given response protocol buffer.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def deserialize_response(self, response_bytestring):
    """Deserialize a response protocol buffer.

    Args:
      response_bytestring: The bytestring serialization of a response protocol
        buffer.

    Returns:
      The response protocol buffer deserialized from the given byte string.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def requests(self):
    """Access the sequence of requests for this scenario.

    Returns:
      A sequence of request protocol buffers.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def response_for_request(self, request):
    """Access the response for a particular request.

    Args:
      request: A request protocol buffer.

    Returns:
      The response protocol buffer appropriate for the given request.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def verify_requests(self, experimental_requests):
    """Verify the requests transmitted through the system under test.

    Args:
      experimental_requests: The request protocol buffers transmitted through
        the system under test.

    Returns:
      True if the requests satisfy this test scenario; False otherwise.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def verify_responses(self, experimental_responses):
    """Verify the responses transmitted through the system under test.

    Args:
      experimental_responses: The response protocol buffers transmitted through
        the system under test.

    Returns:
      True if the responses satisfy this test scenario; False otherwise.
    """
    raise NotImplementedError()


class EmptyScenario(ProtoScenario):
  """A scenario that transmits no protocol buffers in either direction."""

  def method(self):
    return 'DivMany'

  def serialize_request(self, request):
    raise ValueError('This should not be necessary to call!')

  def deserialize_request(self, request_bytestring):
    raise ValueError('This should not be necessary to call!')

  def serialize_response(self, response):
    raise ValueError('This should not be necessary to call!')

  def deserialize_response(self, response_bytestring):
    raise ValueError('This should not be necessary to call!')

  def requests(self):
    return ()

  def response_for_request(self, request):
    raise ValueError('This should not be necessary to call!')

  def verify_requests(self, experimental_requests):
    return not experimental_requests

  def verify_responses(self, experimental_responses):
    return not experimental_responses


class BidirectionallyUnaryScenario(ProtoScenario):
  """A scenario that transmits no protocol buffers in either direction."""

  _DIVIDEND = 59
  _DIVISOR = 7
  _QUOTIENT = 8
  _REMAINDER = 3

  _REQUEST = math_pb2.DivArgs(dividend=_DIVIDEND, divisor=_DIVISOR)
  _RESPONSE = math_pb2.DivReply(quotient=_QUOTIENT, remainder=_REMAINDER)

  def method(self):
    return 'Div'

  def serialize_request(self, request):
    return request.SerializeToString()

  def deserialize_request(self, request_bytestring):
    return math_pb2.DivArgs.FromString(request_bytestring)

  def serialize_response(self, response):
    return response.SerializeToString()

  def deserialize_response(self, response_bytestring):
    return math_pb2.DivReply.FromString(response_bytestring)

  def requests(self):
    return [self._REQUEST]

  def response_for_request(self, request):
    return self._RESPONSE

  def verify_requests(self, experimental_requests):
    return tuple(experimental_requests) == (self._REQUEST,)

  def verify_responses(self, experimental_responses):
    return tuple(experimental_responses) == (self._RESPONSE,)


class BidirectionallyStreamingScenario(ProtoScenario):
  """A scenario that transmits no protocol buffers in either direction."""

  _STREAM_LENGTH = 200
  _REQUESTS = tuple(
      math_pb2.DivArgs(dividend=59 + index, divisor=7 + index)
      for index in range(_STREAM_LENGTH))

  def __init__(self):
    self._lock = threading.Lock()
    self._responses = []

  def method(self):
    return 'DivMany'

  def serialize_request(self, request):
    return request.SerializeToString()

  def deserialize_request(self, request_bytestring):
    return math_pb2.DivArgs.FromString(request_bytestring)

  def serialize_response(self, response):
    return response.SerializeToString()

  def deserialize_response(self, response_bytestring):
    return math_pb2.DivReply.FromString(response_bytestring)

  def requests(self):
    return self._REQUESTS

  def response_for_request(self, request):
    quotient, remainder = divmod(request.dividend, request.divisor)
    response = math_pb2.DivReply(quotient=quotient, remainder=remainder)
    with self._lock:
      self._responses.append(response)
    return response

  def verify_requests(self, experimental_requests):
    return tuple(experimental_requests) == self._REQUESTS

  def verify_responses(self, experimental_responses):
    with self._lock:
      return tuple(experimental_responses) == tuple(self._responses)



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A utility useful in tests of asynchronous, event-driven interfaces."""

import threading

from grpc.framework.foundation import stream


class Callback(stream.Consumer):
  """A utility object useful in tests of asynchronous code."""

  def __init__(self):
    self._condition = threading.Condition()
    self._unary_response = None
    self._streamed_responses = []
    self._completed = False
    self._abortion = None

  def abort(self, abortion):
    with self._condition:
      self._abortion = abortion
      self._condition.notify_all()

  def complete(self, unary_response):
    with self._condition:
      self._unary_response = unary_response
      self._completed = True
      self._condition.notify_all()

  def consume(self, streamed_response):
    with self._condition:
      self._streamed_responses.append(streamed_response)

  def terminate(self):
    with self._condition:
      self._completed = True
      self._condition.notify_all()

  def consume_and_terminate(self, streamed_response):
    with self._condition:
      self._streamed_responses.append(streamed_response)
      self._completed = True
      self._condition.notify_all()

  def block_until_terminated(self):
    with self._condition:
      while self._abortion is None and not self._completed:
        self._condition.wait()

  def response(self):
    with self._condition:
      if self._abortion is None:
        return self._unary_response
      else:
        raise AssertionError('Aborted with abortion "%s"!' % self._abortion)

  def responses(self):
    with self._condition:
      if self._abortion is None:
        return list(self._streamed_responses)
      else:
        raise AssertionError('Aborted with abortion "%s"!' % self._abortion)

  def abortion(self):
    with self._condition:
      return self._abortion



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Tests for _adapter._low."""

import time
import unittest

from grpc._adapter import _low

_STREAM_LENGTH = 300
_TIMEOUT = 5
_AFTER_DELAY = 2
_FUTURE = time.time() + 60 * 60 * 24
_BYTE_SEQUENCE = b'\abcdefghijklmnopqrstuvwxyz0123456789' * 200
_BYTE_SEQUENCE_SEQUENCE = tuple(
    bytes(bytearray((row + column) % 256 for column in range(row)))
    for row in range(_STREAM_LENGTH))


class LonelyClientTest(unittest.TestCase):

  def testLonelyClient(self):
    host = 'nosuchhostexists'
    port = 54321
    method = 'test method'
    deadline = time.time() + _TIMEOUT
    after_deadline = deadline + _AFTER_DELAY
    metadata_tag = object()
    finish_tag = object()

    completion_queue = _low.CompletionQueue()
    channel = _low.Channel('%s:%d' % (host, port), None)
    client_call = _low.Call(channel, method, host, deadline)

    client_call.invoke(completion_queue, metadata_tag, finish_tag)
    first_event = completion_queue.get(after_deadline)
    self.assertIsNotNone(first_event)
    second_event = completion_queue.get(after_deadline)
    self.assertIsNotNone(second_event)
    kinds = [event.kind for event in (first_event, second_event)]
    self.assertItemsEqual(
        (_low.Event.Kind.METADATA_ACCEPTED, _low.Event.Kind.FINISH),
        kinds)

    self.assertIsNone(completion_queue.get(after_deadline))

    completion_queue.stop()
    stop_event = completion_queue.get(_FUTURE)
    self.assertEqual(_low.Event.Kind.STOP, stop_event.kind)


class EchoTest(unittest.TestCase):

  def setUp(self):
    self.host = 'localhost'

    self.server_completion_queue = _low.CompletionQueue()
    self.server = _low.Server(self.server_completion_queue)
    port = self.server.add_http2_addr('[::]:0')
    self.server.start()

    self.client_completion_queue = _low.CompletionQueue()
    self.channel = _low.Channel('%s:%d' % (self.host, port), None)

  def tearDown(self):
    self.server.stop()
    # NOTE(nathaniel): Yep, this is weird; it's a consequence of
    # grpc_server_destroy's being what has the effect of telling the server's
    # completion queue to pump out all pending events/tags immediately rather
    # than gracefully completing all outstanding RPCs while accepting no new
    # ones.
    # TODO(nathaniel): Deallocation of a Python object shouldn't have this kind
    # of observable side effect let alone such an important one.
    del self.server
    self.server_completion_queue.stop()
    self.client_completion_queue.stop()
    while True:
      event = self.server_completion_queue.get(_FUTURE)
      if event is not None and event.kind is _low.Event.Kind.STOP:
        break
    while True:
      event = self.client_completion_queue.get(_FUTURE)
      if event is not None and event.kind is _low.Event.Kind.STOP:
        break
    self.server_completion_queue = None
    self.client_completion_queue = None

  def _perform_echo_test(self, test_data):
    method = 'test method'
    details = 'test details'
    deadline = _FUTURE
    metadata_tag = object()
    finish_tag = object()
    write_tag = object()
    complete_tag = object()
    service_tag = object()
    read_tag = object()
    status_tag = object()

    server_data = []
    client_data = []

    client_call = _low.Call(self.channel, method, self.host, deadline)

    client_call.invoke(self.client_completion_queue, metadata_tag, finish_tag)

    self.server.service(service_tag)
    service_accepted = self.server_completion_queue.get(_FUTURE)
    self.assertIsNotNone(service_accepted)
    self.assertIs(service_accepted.kind, _low.Event.Kind.SERVICE_ACCEPTED)
    self.assertIs(service_accepted.tag, service_tag)
    self.assertEqual(method, service_accepted.service_acceptance.method)
    self.assertEqual(self.host, service_accepted.service_acceptance.host)
    self.assertIsNotNone(service_accepted.service_acceptance.call)
    server_call = service_accepted.service_acceptance.call
    server_call.accept(self.server_completion_queue, finish_tag)
    server_call.premetadata()

    metadata_accepted = self.client_completion_queue.get(_FUTURE)
    self.assertIsNotNone(metadata_accepted)
    self.assertEqual(_low.Event.Kind.METADATA_ACCEPTED, metadata_accepted.kind)
    self.assertEqual(metadata_tag, metadata_accepted.tag)
    # TODO(nathaniel): Test transmission and reception of metadata.

    for datum in test_data:
      client_call.write(datum, write_tag)
      write_accepted = self.client_completion_queue.get(_FUTURE)
      self.assertIsNotNone(write_accepted)
      self.assertIs(write_accepted.kind, _low.Event.Kind.WRITE_ACCEPTED)
      self.assertIs(write_accepted.tag, write_tag)
      self.assertIs(write_accepted.write_accepted, True)

      server_call.read(read_tag)
      read_accepted = self.server_completion_queue.get(_FUTURE)
      self.assertIsNotNone(read_accepted)
      self.assertEqual(_low.Event.Kind.READ_ACCEPTED, read_accepted.kind)
      self.assertEqual(read_tag, read_accepted.tag)
      self.assertIsNotNone(read_accepted.bytes)
      server_data.append(read_accepted.bytes)

      server_call.write(read_accepted.bytes, write_tag)
      write_accepted = self.server_completion_queue.get(_FUTURE)
      self.assertIsNotNone(write_accepted)
      self.assertEqual(_low.Event.Kind.WRITE_ACCEPTED, write_accepted.kind)
      self.assertEqual(write_tag, write_accepted.tag)
      self.assertTrue(write_accepted.write_accepted)

      client_call.read(read_tag)
      read_accepted = self.client_completion_queue.get(_FUTURE)
      self.assertIsNotNone(read_accepted)
      self.assertEqual(_low.Event.Kind.READ_ACCEPTED, read_accepted.kind)
      self.assertEqual(read_tag, read_accepted.tag)
      self.assertIsNotNone(read_accepted.bytes)
      client_data.append(read_accepted.bytes)

    client_call.complete(complete_tag)
    complete_accepted = self.client_completion_queue.get(_FUTURE)
    self.assertIsNotNone(complete_accepted)
    self.assertIs(complete_accepted.kind, _low.Event.Kind.COMPLETE_ACCEPTED)
    self.assertIs(complete_accepted.tag, complete_tag)
    self.assertIs(complete_accepted.complete_accepted, True)

    server_call.read(read_tag)
    read_accepted = self.server_completion_queue.get(_FUTURE)
    self.assertIsNotNone(read_accepted)
    self.assertEqual(_low.Event.Kind.READ_ACCEPTED, read_accepted.kind)
    self.assertEqual(read_tag, read_accepted.tag)
    self.assertIsNone(read_accepted.bytes)

    server_call.status(_low.Status(_low.Code.OK, details), status_tag)
    server_terminal_event_one = self.server_completion_queue.get(_FUTURE)
    server_terminal_event_two = self.server_completion_queue.get(_FUTURE)
    if server_terminal_event_one.kind == _low.Event.Kind.COMPLETE_ACCEPTED:
      status_accepted = server_terminal_event_one
      rpc_accepted = server_terminal_event_two
    else:
      status_accepted = server_terminal_event_two
      rpc_accepted = server_terminal_event_one
    self.assertIsNotNone(status_accepted)
    self.assertIsNotNone(rpc_accepted)
    self.assertEqual(_low.Event.Kind.COMPLETE_ACCEPTED, status_accepted.kind)
    self.assertEqual(status_tag, status_accepted.tag)
    self.assertTrue(status_accepted.complete_accepted)
    self.assertEqual(_low.Event.Kind.FINISH, rpc_accepted.kind)
    self.assertEqual(finish_tag, rpc_accepted.tag)
    self.assertEqual(_low.Status(_low.Code.OK, ''), rpc_accepted.status)

    client_call.read(read_tag)
    client_terminal_event_one = self.client_completion_queue.get(_FUTURE)
    client_terminal_event_two = self.client_completion_queue.get(_FUTURE)
    if client_terminal_event_one.kind == _low.Event.Kind.READ_ACCEPTED:
      read_accepted = client_terminal_event_one
      finish_accepted = client_terminal_event_two
    else:
      read_accepted = client_terminal_event_two
      finish_accepted = client_terminal_event_one
    self.assertIsNotNone(read_accepted)
    self.assertIsNotNone(finish_accepted)
    self.assertEqual(_low.Event.Kind.READ_ACCEPTED, read_accepted.kind)
    self.assertEqual(read_tag, read_accepted.tag)
    self.assertIsNone(read_accepted.bytes)
    self.assertEqual(_low.Event.Kind.FINISH, finish_accepted.kind)
    self.assertEqual(finish_tag, finish_accepted.tag)
    self.assertEqual(_low.Status(_low.Code.OK, details), finish_accepted.status)

    server_timeout_none_event = self.server_completion_queue.get(0)
    self.assertIsNone(server_timeout_none_event)
    client_timeout_none_event = self.client_completion_queue.get(0)
    self.assertIsNone(client_timeout_none_event)

    self.assertSequenceEqual(test_data, server_data)
    self.assertSequenceEqual(test_data, client_data)

  def testNoEcho(self):
    self._perform_echo_test(())

  def testOneByteEcho(self):
    self._perform_echo_test([b'\x07'])

  def testOneManyByteEcho(self):
    self._perform_echo_test([_BYTE_SEQUENCE])

  def testManyOneByteEchoes(self):
    self._perform_echo_test(_BYTE_SEQUENCE)

  def testManyManyByteEchoes(self):
    self._perform_echo_test(_BYTE_SEQUENCE_SEQUENCE)


class CancellationTest(unittest.TestCase):

  def setUp(self):
    self.host = 'localhost'

    self.server_completion_queue = _low.CompletionQueue()
    self.server = _low.Server(self.server_completion_queue)
    port = self.server.add_http2_addr('[::]:0')
    self.server.start()

    self.client_completion_queue = _low.CompletionQueue()
    self.channel = _low.Channel('%s:%d' % (self.host, port), None)

  def tearDown(self):
    self.server.stop()
    del self.server
    self.server_completion_queue.stop()
    self.client_completion_queue.stop()
    while True:
      event = self.server_completion_queue.get(0)
      if event is not None and event.kind is _low.Event.Kind.STOP:
        break
    while True:
      event = self.client_completion_queue.get(0)
      if event is not None and event.kind is _low.Event.Kind.STOP:
        break

  def testCancellation(self):
    method = 'test method'
    deadline = _FUTURE
    metadata_tag = object()
    finish_tag = object()
    write_tag = object()
    service_tag = object()
    read_tag = object()
    test_data = _BYTE_SEQUENCE_SEQUENCE

    server_data = []
    client_data = []

    client_call = _low.Call(self.channel, method, self.host, deadline)

    client_call.invoke(self.client_completion_queue, metadata_tag, finish_tag)

    self.server.service(service_tag)
    service_accepted = self.server_completion_queue.get(_FUTURE)
    server_call = service_accepted.service_acceptance.call

    server_call.accept(self.server_completion_queue, finish_tag)
    server_call.premetadata()

    metadata_accepted = self.client_completion_queue.get(_FUTURE)
    self.assertIsNotNone(metadata_accepted)

    for datum in test_data:
      client_call.write(datum, write_tag)
      write_accepted = self.client_completion_queue.get(_FUTURE)

      server_call.read(read_tag)
      read_accepted = self.server_completion_queue.get(_FUTURE)
      server_data.append(read_accepted.bytes)

      server_call.write(read_accepted.bytes, write_tag)
      write_accepted = self.server_completion_queue.get(_FUTURE)
      self.assertIsNotNone(write_accepted)

      client_call.read(read_tag)
      read_accepted = self.client_completion_queue.get(_FUTURE)
      client_data.append(read_accepted.bytes)

    client_call.cancel()
    # cancel() is idempotent.
    client_call.cancel()
    client_call.cancel()
    client_call.cancel()

    server_call.read(read_tag)

    server_terminal_event_one = self.server_completion_queue.get(_FUTURE)
    server_terminal_event_two = self.server_completion_queue.get(_FUTURE)
    if server_terminal_event_one.kind == _low.Event.Kind.READ_ACCEPTED:
      read_accepted = server_terminal_event_one
      rpc_accepted = server_terminal_event_two
    else:
      read_accepted = server_terminal_event_two
      rpc_accepted = server_terminal_event_one
    self.assertIsNotNone(read_accepted)
    self.assertIsNotNone(rpc_accepted)
    self.assertEqual(_low.Event.Kind.READ_ACCEPTED, read_accepted.kind)
    self.assertIsNone(read_accepted.bytes)
    self.assertEqual(_low.Event.Kind.FINISH, rpc_accepted.kind)
    self.assertEqual(_low.Status(_low.Code.CANCELLED, ''), rpc_accepted.status)

    finish_event = self.client_completion_queue.get(_FUTURE)
    self.assertEqual(_low.Event.Kind.FINISH, finish_event.kind)
    self.assertEqual(_low.Status(_low.Code.CANCELLED, ''), finish_event.status)

    server_timeout_none_event = self.server_completion_queue.get(0)
    self.assertIsNone(server_timeout_none_event)
    client_timeout_none_event = self.client_completion_queue.get(0)
    self.assertIsNone(client_timeout_none_event)

    self.assertSequenceEqual(test_data, server_data)
    self.assertSequenceEqual(test_data, client_data)


class ExpirationTest(unittest.TestCase):

  @unittest.skip('TODO(nathaniel): Expiration test!')
  def testExpiration(self):
    pass


if __name__ == '__main__':
  unittest.main()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""State and behavior for ingestion during an operation."""

import abc
import collections

from grpc.framework.base import _constants
from grpc.framework.base import _interfaces
from grpc.framework.base import exceptions
from grpc.framework.base import interfaces
from grpc.framework.foundation import abandonment
from grpc.framework.foundation import callable_util
from grpc.framework.foundation import stream

_CREATE_CONSUMER_EXCEPTION_LOG_MESSAGE = 'Exception initializing ingestion!'
_CONSUME_EXCEPTION_LOG_MESSAGE = 'Exception during ingestion!'


class _ConsumerCreation(collections.namedtuple(
    '_ConsumerCreation', ('consumer', 'remote_error', 'abandoned'))):
  """A sum type for the outcome of ingestion initialization.

  Either consumer will be non-None, remote_error will be True, or abandoned will
  be True.

  Attributes:
    consumer: A stream.Consumer for ingesting payloads.
    remote_error: A boolean indicating that the consumer could not be created
      due to an error on the remote side of the operation.
    abandoned: A boolean indicating that the consumer creation was abandoned.
  """


class _EmptyConsumer(stream.Consumer):
  """A no-operative stream.Consumer that ignores all inputs and calls."""

  def consume(self, value):
    """See stream.Consumer.consume for specification."""

  def terminate(self):
    """See stream.Consumer.terminate for specification."""

  def consume_and_terminate(self, value):
    """See stream.Consumer.consume_and_terminate for specification."""


class _ConsumerCreator(object):
  """Common specification of different consumer-creating behavior."""
  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def create_consumer(self, requirement):
    """Creates the stream.Consumer to which customer payloads will be delivered.

    Any exceptions raised by this method should be attributed to and treated as
    defects in the serviced or servicer code called by this method.

    Args:
      requirement: A value required by this _ConsumerCreator for consumer
        creation.

    Returns:
      A _ConsumerCreation describing the result of consumer creation.
    """
    raise NotImplementedError()


class _FrontConsumerCreator(_ConsumerCreator):
  """A _ConsumerCreator appropriate for front-side use."""

  def __init__(self, subscription, operation_context):
    """Constructor.

    Args:
      subscription: The serviced's interfaces.ServicedSubscription for the
        operation.
      operation_context: The interfaces.OperationContext object for the
        operation.
    """
    self._subscription = subscription
    self._operation_context = operation_context

  def create_consumer(self, requirement):
    """See _ConsumerCreator.create_consumer for specification."""
    if self._subscription.kind is interfaces.ServicedSubscription.Kind.FULL:
      try:
        return _ConsumerCreation(
            self._subscription.ingestor.consumer(self._operation_context),
            False, False)
      except abandonment.Abandoned:
        return _ConsumerCreation(None, False, True)
    else:
      return _ConsumerCreation(_EmptyConsumer(), False, False)


class _BackConsumerCreator(_ConsumerCreator):
  """A _ConsumerCreator appropriate for back-side use."""

  def __init__(self, servicer, operation_context, emission_consumer):
    """Constructor.

    Args:
      servicer: The interfaces.Servicer that will service the operation.
      operation_context: The interfaces.OperationContext object for the
        operation.
      emission_consumer: The stream.Consumer object to which payloads emitted
        from the operation will be passed.
    """
    self._servicer = servicer
    self._operation_context = operation_context
    self._emission_consumer = emission_consumer

  def create_consumer(self, requirement):
    """See _ConsumerCreator.create_consumer for full specification.

    Args:
      requirement: The name of the Servicer method to be called during this
        operation.

    Returns:
      A _ConsumerCreation describing the result of consumer creation.
    """
    try:
      return _ConsumerCreation(
          self._servicer.service(
              requirement, self._operation_context, self._emission_consumer),
          False, False)
    except exceptions.NoSuchMethodError:
      return _ConsumerCreation(None, True, False)
    except abandonment.Abandoned:
      return _ConsumerCreation(None, False, True)


class _WrappedConsumer(object):
  """Wraps a consumer to catch the exceptions that it is allowed to throw."""

  def __init__(self, consumer):
    """Constructor.

    Args:
      consumer: A stream.Consumer that may raise abandonment.Abandoned from any
        of its methods.
    """
    self._consumer = consumer

  def moar(self, payload, complete):
    """Makes progress with the wrapped consumer.

    This method catches all exceptions allowed to be thrown by the wrapped
    consumer. Any exceptions raised by this method should be blamed on the
    customer-supplied consumer.

    Args:
      payload: A customer-significant payload object. May be None only if
        complete is True.
      complete: Whether or not the end of the payload sequence has been reached.
        Must be True if payload is None.

    Returns:
      True if the wrapped consumer made progress or False if the wrapped
        consumer raised abandonment.Abandoned to indicate its abandonment of
        progress.
    """
    try:
      if payload is None:
        self._consumer.terminate()
      elif complete:
        self._consumer.consume_and_terminate(payload)
      else:
        self._consumer.consume(payload)
      return True
    except abandonment.Abandoned:
      return False


class _IngestionManager(_interfaces.IngestionManager):
  """An implementation of _interfaces.IngestionManager."""

  def __init__(
      self, lock, pool, consumer_creator, failure_outcome, termination_manager,
      transmission_manager):
    """Constructor.

    Args:
      lock: The operation-wide lock.
      pool: A thread pool in which to execute customer code.
      consumer_creator: A _ConsumerCreator wrapping the portion of customer code
        that when called returns the stream.Consumer with which the customer
        code will ingest payload values.
      failure_outcome: Whichever one of
        interfaces.Outcome.SERVICED_FAILURE or
        interfaces.Outcome.SERVICER_FAILURE describes local failure of
        customer code.
      termination_manager: The _interfaces.TerminationManager for the operation.
      transmission_manager: The _interfaces.TransmissionManager for the
        operation.
    """
    self._lock = lock
    self._pool = pool
    self._consumer_creator = consumer_creator
    self._failure_outcome = failure_outcome
    self._termination_manager = termination_manager
    self._transmission_manager = transmission_manager
    self._expiration_manager = None

    self._wrapped_ingestion_consumer = None
    self._pending_ingestion = []
    self._ingestion_complete = False
    self._processing = False

  def set_expiration_manager(self, expiration_manager):
    self._expiration_manager = expiration_manager

  def _abort_internal_only(self):
    self._wrapped_ingestion_consumer = None
    self._pending_ingestion = None

  def _abort_and_notify(self, outcome):
    self._abort_internal_only()
    self._termination_manager.abort(outcome)
    self._transmission_manager.abort(outcome)
    self._expiration_manager.abort()

  def _next(self):
    """Computes the next step for ingestion.

    Returns:
      A payload, complete, continue triplet indicating what payload (if any) is
        available to feed into customer code, whether or not the sequence of
        payloads has terminated, and whether or not there is anything
        immediately actionable to call customer code to do.
    """
    if self._pending_ingestion is None:
      return None, False, False
    elif self._pending_ingestion:
      payload = self._pending_ingestion.pop(0)
      complete = self._ingestion_complete and not self._pending_ingestion
      return payload, complete, True
    elif self._ingestion_complete:
      return None, True, True
    else:
      return None, False, False

  def _process(self, wrapped_ingestion_consumer, payload, complete):
    """A method to call to execute customer code.

    This object's lock must *not* be held when calling this method.

    Args:
      wrapped_ingestion_consumer: The _WrappedConsumer with which to pass
        payloads to customer code.
      payload: A customer payload. May be None only if complete is True.
      complete: Whether or not the sequence of payloads to pass to the customer
        has concluded.
    """
    while True:
      consumption_outcome = callable_util.call_logging_exceptions(
          wrapped_ingestion_consumer.moar, _CONSUME_EXCEPTION_LOG_MESSAGE,
          payload, complete)
      if consumption_outcome.exception is None:
        if consumption_outcome.return_value:
          with self._lock:
            if complete:
              self._pending_ingestion = None
              self._termination_manager.ingestion_complete()
              return
            else:
              payload, complete, moar = self._next()
              if not moar:
                self._processing = False
                return
        else:
          with self._lock:
            if self._pending_ingestion is not None:
              self._abort_and_notify(self._failure_outcome)
            self._processing = False
            return
      else:
        with self._lock:
          self._abort_and_notify(self._failure_outcome)
          self._processing = False
          return

  def start(self, requirement):
    if self._pending_ingestion is not None:
      def initialize():
        consumer_creation_outcome = callable_util.call_logging_exceptions(
            self._consumer_creator.create_consumer,
            _CREATE_CONSUMER_EXCEPTION_LOG_MESSAGE, requirement)
        if consumer_creation_outcome.return_value is None:
          with self._lock:
            self._abort_and_notify(self._failure_outcome)
            self._processing = False
        elif consumer_creation_outcome.return_value.remote_error:
          with self._lock:
            self._abort_and_notify(interfaces.Outcome.RECEPTION_FAILURE)
            self._processing = False
        elif consumer_creation_outcome.return_value.abandoned:
          with self._lock:
            if self._pending_ingestion is not None:
              self._abort_and_notify(self._failure_outcome)
            self._processing = False
        else:
          wrapped_ingestion_consumer = _WrappedConsumer(
              consumer_creation_outcome.return_value.consumer)
          with self._lock:
            self._wrapped_ingestion_consumer = wrapped_ingestion_consumer
            payload, complete, moar = self._next()
            if not moar:
              self._processing = False
              return

          self._process(wrapped_ingestion_consumer, payload, complete)

      self._pool.submit(
          callable_util.with_exceptions_logged(
              initialize, _constants.INTERNAL_ERROR_LOG_MESSAGE))
      self._processing = True

  def consume(self, payload):
    if self._ingestion_complete:
      self._abort_and_notify(self._failure_outcome)
    elif self._pending_ingestion is not None:
      if self._processing:
        self._pending_ingestion.append(payload)
      else:
        self._pool.submit(
            callable_util.with_exceptions_logged(
                self._process, _constants.INTERNAL_ERROR_LOG_MESSAGE),
            self._wrapped_ingestion_consumer, payload, False)
        self._processing = True

  def terminate(self):
    if self._ingestion_complete:
      self._abort_and_notify(self._failure_outcome)
    else:
      self._ingestion_complete = True
      if self._pending_ingestion is not None and not self._processing:
        self._pool.submit(
            callable_util.with_exceptions_logged(
                self._process, _constants.INTERNAL_ERROR_LOG_MESSAGE),
            self._wrapped_ingestion_consumer, None, True)
        self._processing = True

  def consume_and_terminate(self, payload):
    if self._ingestion_complete:
      self._abort_and_notify(self._failure_outcome)
    else:
      self._ingestion_complete = True
      if self._pending_ingestion is not None:
        if self._processing:
          self._pending_ingestion.append(payload)
        else:
          self._pool.submit(
              callable_util.with_exceptions_logged(
                  self._process, _constants.INTERNAL_ERROR_LOG_MESSAGE),
              self._wrapped_ingestion_consumer, payload, True)
          self._processing = True

  def abort(self):
    """See _interfaces.IngestionManager.abort for specification."""
    self._abort_internal_only()


def front_ingestion_manager(
    lock, pool, subscription, termination_manager, transmission_manager,
    operation_context):
  """Creates an IngestionManager appropriate for front-side use.

  Args:
    lock: The operation-wide lock.
    pool: A thread pool in which to execute customer code.
    subscription: A interfaces.ServicedSubscription indicating the
      customer's interest in the results of the operation.
    termination_manager: The _interfaces.TerminationManager for the operation.
    transmission_manager: The _interfaces.TransmissionManager for the
      operation.
    operation_context: A interfaces.OperationContext for the operation.

  Returns:
    An IngestionManager appropriate for front-side use.
  """
  ingestion_manager = _IngestionManager(
      lock, pool, _FrontConsumerCreator(subscription, operation_context),
      interfaces.Outcome.SERVICED_FAILURE, termination_manager,
      transmission_manager)
  ingestion_manager.start(None)
  return ingestion_manager


def back_ingestion_manager(
    lock, pool, servicer, termination_manager, transmission_manager,
    operation_context, emission_consumer):
  """Creates an IngestionManager appropriate for back-side use.

  Args:
    lock: The operation-wide lock.
    pool: A thread pool in which to execute customer code.
    servicer: A interfaces.Servicer for servicing the operation.
    termination_manager: The _interfaces.TerminationManager for the operation.
    transmission_manager: The _interfaces.TransmissionManager for the
      operation.
    operation_context: A interfaces.OperationContext for the operation.
    emission_consumer: The _interfaces.EmissionConsumer for the operation.

  Returns:
    An IngestionManager appropriate for back-side use.
  """
  ingestion_manager = _IngestionManager(
      lock, pool, _BackConsumerCreator(
          servicer, operation_context, emission_consumer),
      interfaces.Outcome.SERVICER_FAILURE, termination_manager,
      transmission_manager)
  return ingestion_manager



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# TODO(nathaniel): Expand this test coverage.

"""Test of the GRPC-backed ForeLink and RearLink."""

import unittest

from grpc.early_adopter import implementations
from grpc.framework.alpha import utilities
from grpc._junkdrawer import math_pb2

SERVICE_NAME = 'math.Math'

DIV = 'Div'
DIV_MANY = 'DivMany'
FIB = 'Fib'
SUM = 'Sum'

def _fibbonacci(limit):
  left, right = 0, 1
  for _ in xrange(limit):
    yield left
    left, right = right, left + right


def _div(request, unused_context):
  return math_pb2.DivReply(
      quotient=request.dividend / request.divisor,
      remainder=request.dividend % request.divisor)


def _div_many(request_iterator, unused_context):
  for request in request_iterator:
    yield math_pb2.DivReply(
        quotient=request.dividend / request.divisor,
        remainder=request.dividend % request.divisor)


def _fib(request, unused_context):
  for number in _fibbonacci(request.limit):
    yield math_pb2.Num(num=number)


def _sum(request_iterator, unused_context):
  accumulation = 0
  for request in request_iterator:
    accumulation += request.num
  return math_pb2.Num(num=accumulation)


_INVOCATION_DESCRIPTIONS = {
    DIV: utilities.unary_unary_invocation_description(
        math_pb2.DivArgs.SerializeToString, math_pb2.DivReply.FromString),
    DIV_MANY: utilities.stream_stream_invocation_description(
        math_pb2.DivArgs.SerializeToString, math_pb2.DivReply.FromString),
    FIB: utilities.unary_stream_invocation_description(
        math_pb2.FibArgs.SerializeToString, math_pb2.Num.FromString),
    SUM: utilities.stream_unary_invocation_description(
        math_pb2.Num.SerializeToString, math_pb2.Num.FromString),
}

_SERVICE_DESCRIPTIONS = {
    DIV: utilities.unary_unary_service_description(
        _div, math_pb2.DivArgs.FromString,
        math_pb2.DivReply.SerializeToString),
    DIV_MANY: utilities.stream_stream_service_description(
        _div_many, math_pb2.DivArgs.FromString,
        math_pb2.DivReply.SerializeToString),
    FIB: utilities.unary_stream_service_description(
        _fib, math_pb2.FibArgs.FromString, math_pb2.Num.SerializeToString),
    SUM: utilities.stream_unary_service_description(
        _sum, math_pb2.Num.FromString, math_pb2.Num.SerializeToString),
}

_TIMEOUT = 3


class EarlyAdopterImplementationsTest(unittest.TestCase):

  def setUp(self):
    self.server = implementations.insecure_server(
        SERVICE_NAME, _SERVICE_DESCRIPTIONS, 0)
    self.server.start()
    port = self.server.port()
    self.stub = implementations.insecure_stub(
        SERVICE_NAME, _INVOCATION_DESCRIPTIONS, 'localhost', port)

  def tearDown(self):
    self.server.stop()

  def testUpAndDown(self):
    with self.stub:
      pass

  def testUnaryUnary(self):
    divisor = 59
    dividend = 973
    expected_quotient = dividend / divisor
    expected_remainder = dividend % divisor

    with self.stub:
      response = self.stub.Div(
          math_pb2.DivArgs(divisor=divisor, dividend=dividend), _TIMEOUT)
      self.assertEqual(expected_quotient, response.quotient)
      self.assertEqual(expected_remainder, response.remainder)

  def testUnaryStream(self):
    stream_length = 43

    with self.stub:
      response_iterator = self.stub.Fib(
          math_pb2.FibArgs(limit=stream_length), _TIMEOUT)
      numbers = tuple(response.num for response in response_iterator)
      for early, middle, later in zip(numbers, numbers[:1], numbers[:2]):
        self.assertEqual(early + middle, later)
      self.assertEqual(stream_length, len(numbers))

  def testStreamUnary(self):
    stream_length = 127

    with self.stub:
      response_future = self.stub.Sum.async(
          (math_pb2.Num(num=index) for index in range(stream_length)),
          _TIMEOUT)
      self.assertEqual(
          (stream_length * (stream_length - 1)) / 2,
          response_future.result().num)

  def testStreamStream(self):
    stream_length = 179
    divisor_offset = 71
    dividend_offset = 1763

    with self.stub:
      response_iterator = self.stub.DivMany(
          (math_pb2.DivArgs(
               divisor=divisor_offset + index,
               dividend=dividend_offset + index)
           for index in range(stream_length)),
          _TIMEOUT)
      for index, response in enumerate(response_iterator):
        self.assertEqual(
            (dividend_offset + index) / (divisor_offset + index),
            response.quotient)
        self.assertEqual(
            (dividend_offset + index) % (divisor_offset + index),
            response.remainder)
      self.assertEqual(stream_length, index + 1)


if __name__ == '__main__':
  unittest.main()



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Private interfaces implemented by data sets used in Face-layer tests."""

import abc

# interfaces is referenced from specification in this module.
from grpc.framework.face import interfaces as face_interfaces  # pylint: disable=unused-import
from grpc.framework.face.testing import interfaces


class UnaryUnaryTestMethodImplementation(interfaces.Method):
  """A controllable implementation of a unary-unary RPC method."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def service(self, request, response_callback, context, control):
    """Services an RPC that accepts one message and produces one message.

    Args:
      request: The single request message for the RPC.
      response_callback: A callback to be called to accept the response message
        of the RPC.
      context: An face_interfaces.RpcContext object.
      control: A test_control.Control to control execution of this method.

    Raises:
      abandonment.Abandoned: May or may not be raised when the RPC has been
        aborted.
    """
    raise NotImplementedError()


class UnaryUnaryTestMessages(object):
  """A type for unary-request-unary-response message pairings."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def request(self):
    """Affords a request message.

    Implementations of this method should return a different message with each
    call so that multiple test executions of the test method may be made with
    different inputs.

    Returns:
      A request message.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def verify(self, request, response, test_case):
    """Verifies that the computed response matches the given request.

    Args:
      request: A request message.
      response: A response message.
      test_case: A unittest.TestCase object affording useful assertion methods.

    Raises:
      AssertionError: If the request and response do not match, indicating that
        there was some problem executing the RPC under test.
    """
    raise NotImplementedError()


class UnaryStreamTestMethodImplementation(interfaces.Method):
  """A controllable implementation of a unary-stream RPC method."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def service(self, request, response_consumer, context, control):
    """Services an RPC that takes one message and produces a stream of messages.

    Args:
      request: The single request message for the RPC.
      response_consumer: A stream.Consumer to be called to accept the response
        messages of the RPC.
      context: A face_interfaces.RpcContext object.
      control: A test_control.Control to control execution of this method.

    Raises:
      abandonment.Abandoned: May or may not be raised when the RPC has been
        aborted.
    """
    raise NotImplementedError()


class UnaryStreamTestMessages(object):
  """A type for unary-request-stream-response message pairings."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def request(self):
    """Affords a request message.

    Implementations of this method should return a different message with each
    call so that multiple test executions of the test method may be made with
    different inputs.

    Returns:
      A request message.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def verify(self, request, responses, test_case):
    """Verifies that the computed responses match the given request.

    Args:
      request: A request message.
      responses: A sequence of response messages.
      test_case: A unittest.TestCase object affording useful assertion methods.

    Raises:
      AssertionError: If the request and responses do not match, indicating that
        there was some problem executing the RPC under test.
    """
    raise NotImplementedError()


class StreamUnaryTestMethodImplementation(interfaces.Method):
  """A controllable implementation of a stream-unary RPC method."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def service(self, response_callback, context, control):
    """Services an RPC that takes a stream of messages and produces one message.

    Args:
      response_callback: A callback to be called to accept the response message
        of the RPC.
      context: A face_interfaces.RpcContext object.
      control: A test_control.Control to control execution of this method.

    Returns:
      A stream.Consumer with which to accept the request messages of the RPC.
        The consumer returned from this method may or may not be invoked to
        completion: in the case of RPC abortion, RPC Framework will simply stop
        passing messages to this object. Implementations must not assume that
        this object will be called to completion of the request stream or even
        called at all.

    Raises:
      abandonment.Abandoned: May or may not be raised when the RPC has been
        aborted.
    """
    raise NotImplementedError()


class StreamUnaryTestMessages(object):
  """A type for stream-request-unary-response message pairings."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def requests(self):
    """Affords a sequence of request messages.

    Implementations of this method should return a different sequences with each
    call so that multiple test executions of the test method may be made with
    different inputs.

    Returns:
      A sequence of request messages.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def verify(self, requests, response, test_case):
    """Verifies that the computed response matches the given requests.

    Args:
      requests: A sequence of request messages.
      response: A response message.
      test_case: A unittest.TestCase object affording useful assertion methods.

    Raises:
      AssertionError: If the requests and response do not match, indicating that
        there was some problem executing the RPC under test.
    """
    raise NotImplementedError()


class StreamStreamTestMethodImplementation(interfaces.Method):
  """A controllable implementation of a stream-stream RPC method."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def service(self, response_consumer, context, control):
    """Services an RPC that accepts and produces streams of messages.

    Args:
      response_consumer: A stream.Consumer to be called to accept the response
        messages of the RPC.
      context: A face_interfaces.RpcContext object.
      control: A test_control.Control to control execution of this method.

    Returns:
      A stream.Consumer with which to accept the request messages of the RPC.
        The consumer returned from this method may or may not be invoked to
        completion: in the case of RPC abortion, RPC Framework will simply stop
        passing messages to this object. Implementations must not assume that
        this object will be called to completion of the request stream or even
        called at all.

    Raises:
      abandonment.Abandoned: May or may not be raised when the RPC has been
        aborted.
    """
    raise NotImplementedError()


class StreamStreamTestMessages(object):
  """A type for stream-request-stream-response message pairings."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def requests(self):
    """Affords a sequence of request messages.

    Implementations of this method should return a different sequences with each
    call so that multiple test executions of the test method may be made with
    different inputs.

    Returns:
      A sequence of request messages.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def verify(self, requests, responses, test_case):
    """Verifies that the computed response matches the given requests.

    Args:
      requests: A sequence of request messages.
      responses: A sequence of response messages.
      test_case: A unittest.TestCase object affording useful assertion methods.

    Raises:
      AssertionError: If the requests and responses do not match, indicating
        that there was some problem executing the RPC under test.
    """
    raise NotImplementedError()


class TestService(object):
  """A specification of implemented RPC methods to use in tests."""

  __metaclass__ = abc.ABCMeta

  @abc.abstractmethod
  def name(self):
    """Identifies the RPC service name used during the test.

    Returns:
      The RPC service name to be used for the test.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def unary_unary_scenarios(self):
    """Affords unary-request-unary-response test methods and their messages.

    Returns:
      A dict from method name to pair. The first element of the pair
        is a UnaryUnaryTestMethodImplementation object and the second element
        is a sequence of UnaryUnaryTestMethodMessages objects.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def unary_stream_scenarios(self):
    """Affords unary-request-stream-response test methods and their messages.

    Returns:
      A dict from method name to pair. The first element of the pair is a
        UnaryStreamTestMethodImplementation object and the second element is a
        sequence of UnaryStreamTestMethodMessages objects.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def stream_unary_scenarios(self):
    """Affords stream-request-unary-response test methods and their messages.

    Returns:
      A dict from method name to pair. The first element of the pair is a
        StreamUnaryTestMethodImplementation object and the second element is a
        sequence of StreamUnaryTestMethodMessages objects.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def stream_stream_scenarios(self):
    """Affords stream-request-stream-response test methods and their messages.

    Returns:
      A dict from method name to pair. The first element of the pair is a
        StreamStreamTestMethodImplementation object and the second element is a
        sequence of StreamStreamTestMethodMessages objects.
    """
    raise NotImplementedError()
